# -*- coding: utf-8 -*-

import locale
import itertools
import csv

from datetime import datetime

from contextlib import contextmanager
from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine, generate_transaction_id


def take(iterable, n):
    """Return first n items of the iterable as a list."""
    return list(itertools.islice(iterable, n))


def drop(iterable, n):
    """Drop first n items of the iterable and return result as a list."""
    return list(itertools.islice(iterable, n, None))


def head(iterable):
    """Return first element of the iterable."""
    return take(iterable, 1)[0]


@contextmanager
def scoped_setlocale(category, loc=None):
    """Scoped version of locale.setlocale()"""
    orig = locale.getlocale(category)
    try:
        yield locale.setlocale(category, loc)
    finally:
        locale.setlocale(category, orig)


def atof(string, loc=None):
    """Locale aware atof function for our parser."""
    with scoped_setlocale(locale.LC_NUMERIC, loc):
        return locale.atof(string)


class PayPalStatementParser(StatementParser):
    bank_id = 'PayPal'
    date_format = '%Y/%m/%d'
    valid_header = [
        u"Date",
        u"Time",
        u"Time Zone",
        u"Name",
        u"Type",
        u"Status",
        u"Currency",
        u"Gross",
        u"Fee",
        u"Net",
        u"From Email Address",
        u"To Email Address",
        u"Transaction ID",
        u"Counterparty Status",
        u"Address Status",
        u"Item Title",
        u"Item ID",
        u"Shipping and Handling Amount",
        u"Insurance Amount",
        u"Sales Tax",
        u"Option 1 Name",
        u"Option 1 Value",
        u"Option 2 Name",
        u"Option 2 Value",
        u"Auction Site",
        u"Buyer ID",
        u"Item URL",
        u"Closing Date",
        u"Escrow Id",
        u"Invoice Id",
        u"Reference Txn ID",
        u"Invoice Number",
        u"Custom Number",
        u"Receipt ID",
        u"Balance",
        u"Address Line 1",
        u"Address Line 2/District/Neighborhood",
        u"Town/City",
        u"State/Province/Region/County/Territory/Prefecture/Republic",
        u"Zip/Postal Code",
        u"Country",
        u"Contact Phone Number",
        u"",
    ]

    def __init__(self, fin, account_id, currency, encoding=None, locale=None, analyze=False):
        self.account_id = account_id
        self.currency = currency
        self.locale = locale
        self.encoding = encoding
        self.analyze = analyze

        with open(fin, 'r', encoding=self.encoding) as f:
            self.lines = f.readlines()

        self.validate()
        self.statement = Statement(
            bank_id=self.bank_id,
            account_id=self.account_id,
            currency=self.currency
        )

    @property
    def reader(self):
        return csv.reader(self.lines, delimiter=',', quotechar='"', quoting=csv.QUOTE_ALL)

    @property
    def header(self):
        return [c.strip() for c in head(self.reader)]

    @property
    def rows(self):
        rs = drop(self.reader, 1)
        currency_idx = self.valid_header.index("Currency")
        return [r for r in rs if r[currency_idx] == self.currency]

    def validate(self):
        """
        Validate to ensure csv has the same header we expect.
        """

        expected = self.valid_header
        actual = self.header
        if expected != actual:
            msg = "\n".join([
                "Header template doesn't match:",
                "expected: %s" % expected,
                "actual  : %s" % actual
            ])
            raise ValueError(msg)

    def split_records(self):
        for row in self.rows:
            yield row

    def parse_record(self, row):

        id_idx = self.valid_header.index("Transaction ID")
        date_idx = self.valid_header.index("Date")
        memo_idx = self.valid_header.index("Name")
        refnum_idx = self.valid_header.index("Reference Txn ID")
        amount_idx = self.valid_header.index("Gross")
        payee_idx = self.valid_header.index("To Email Address")
        title_idx = self.valid_header.index("Item Title")

        stmt_line = StatementLine()
        stmt_line.id = row[id_idx]
        stmt_line.date = datetime.strptime(row[date_idx], self.date_format)
        stmt_line.memo = row[memo_idx]

        if self.analyze:
            memo_parts = [
                row[memo_idx]
            ]

            payee = row[payee_idx]
            if payee and (payee.lower() == 'steamgameseu@steampowered.com'):
                memo_parts.append(row[title_idx])

            stmt_line.memo = ' / '.join(filter(bool, memo_parts))

        stmt_line.refnum = row[refnum_idx]
        stmt_line.amount = atof(row[amount_idx].replace(" ", ""), self.locale)

        return stmt_line


def parse_bool(value):
    if value in ('True', 'true', '1'):
        return True
    if value in ('False', 'false', '0'):
        return False
    raise ValueError("Can't parse boolean value: %s" % value)


class PayPalPlugin(Plugin):
    def get_parser(self, fin):
        kwargs = {
            'encoding': 'iso8859-1',
        }
        if self.settings:
            if 'account_id' in self.settings:
                kwargs['account_id'] = self.settings.get('account_id')
            if 'currency' in self.settings:
                kwargs['currency'] = self.settings.get('currency')
            if 'locale' in self.settings:
                kwargs['locale'] = self.settings.get('locale')
            if 'encoding' in self.settings:
                kwargs['encoding'] = self.settings.get('encoding')
            if 'analyze' in self.settings:
                kwargs['analyze'] = parse_bool(self.settings.get('analyze'))
        return PayPalStatementParser(fin, **kwargs)



#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.


def clean_multiple_whitespaces(uncleaned_string):
    """Clean a string from multiple consecutive white spaces."""
    return ' '.join(uncleaned_string.split())


def fix_amount_string(amount):
    """Replace »,« with ».« to make the amount parseable."""
    return amount.replace('.', '').replace(',', '.')

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



# -*- coding: utf-8 -*-

import re
import itertools
import logging
import locale
import os

from xlrd import open_workbook

from contextlib import contextmanager
from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine, generate_transaction_id


def take(n, iterable):
    """Return first n items of the iterable as a list."""
    return list(itertools.islice(iterable, n))


@contextmanager
def scoped_setlocale(category, loc=None):
    """Scoped version of locale.setlocale()"""
    orig = locale.getlocale(category)
    try:
        yield locale.setlocale(category, loc)
    finally:
        locale.setlocale(category, orig)


def atof(string, loc=None):
    """Locale aware atof function for our parser."""
    with scoped_setlocale(locale.LC_NUMERIC, loc):
        return locale.atof(string)


class LfsStatementParser(StatementParser):
    date_format = '%Y-%m-%d'

    footer_regexps = [
        '^Datum:  -',
        '^Datum: ([0-9]{4}-[0-9]{2}-[0-9]{2}) - ([0-9]{4}-[0-9]{2}-[0-9]{2})$'
    ]

    def __init__(self, fin, locale=None, account_id=None, bank_id=None, currency=None):
        """
        Create a new LfsStatementParser instance.

        :param fin: filename to create parser for
        """
        
        if account_id is None:
            account_id = input("Account ID:")
        assert account_id

        if bank_id is None:
            bank_id = input("Bank ID:")
        assert bank_id

        if currency is None:
            currency = input("Currency:")
        assert currency

        self.locale = locale
        self.account_id = account_id
        self.bank_id = bank_id
        self.currency = currency

        self.workbook = open_workbook(filename=fin)
        self.sheet = self.workbook.sheet_by_index(0)
        self.validate()

        self.statement = self.parse_statement()

    def validate(self):
        """
        Naive validation to make sure that the document is structured the way it was
        when this parser was written.

        :raises ValueError if workbook has invalid format
        """

        try:
            self._validate()
        except AssertionError as e:
            raise ValueError(e)

    def parse_float(self, value):
        if isinstance(value, float):
            return value
        return atof(value, self.locale)

    def _validate(self):
        logging.info('Verifying file structure.')

        rows = list(self.sheet.get_rows())

        logging.info('Verifying that at least 2 rows.')
        assert len(rows) >= 2

        logging.info('Verifying that every row has 5 cells.')
        assert type(rows) == list
        for row in rows:
            assert len(row) == 5

        logging.info('Verifying that every cell has a value.')
        rows = [[c.value for c in row] for row in rows]

        logging.info('Verifying that first row is info header.')
        info_header_row = rows[0]
        assert info_header_row[0].startswith('Kontoutdrag')

        logging.info('Verifying that second row is statements header.')
        statement_header_row = rows[1]
        assert ['Bokf.dat', 'Trans.dat', 'Text', 'Insï¿½ttning/Uttag', 'Behï¿½llning'] == statement_header_row

        logging.info('Verified file structure - OK!')

    def sheet_rows(self):
        return [[c.value for c in row] for row in self.sheet.get_rows()]

    def parse_statement(self):
        statement = Statement()

        statement.account_id = self.account_id
        statement.bank_id = self.bank_id
        statement.currency = self.currency

        rows = self.sheet_rows()
        for r in rows[2:]:
            bokf_date, _, _, _, balance = r

            # first row will be our start balance & date
            first_row = (statement.start_date is None) or (statement.start_balance is None)
            if first_row:
                statement.start_date = self.parse_datetime(bokf_date)
                statement.start_balance = self.parse_float(balance)

            # last row will be our end balance & date
            statement.end_balance = self.parse_float(balance)
            statement.end_date = self.parse_datetime(bokf_date)

        #
        # Use the fact that first cell contains the statement date as a suffix e.g. 'Kontoutdrag - 2018-01-04'
        #
        info_header_row = rows[0]
        m = re.match('Kontoutdrag *- *([0-9]{4}-[0-9]{2}-[0-9]{2})$', info_header_row[0])
        if m:
            stmt_date, = m.groups()
            statement.end_date = self.parse_datetime(stmt_date)

        return statement

    def split_records(self):
        rows = self.sheet_rows()

        # Skip first 2 rows. Headers they are.
        for row in itertools.islice(rows, 2, None):
            yield row

    def parse_record(self, row):
        stmt_line = StatementLine()
        stmt_line.date = self.parse_datetime(row[0])
        stmt_line.date_user = self.parse_datetime(row[1])
        stmt_line.memo = row[2]
        stmt_line.amount = self.parse_float(row[3])
        stmt_line.id = generate_transaction_id(stmt_line)
        return stmt_line


def parse_bool(value):
    if value in ('True', 'true', '1'):
        return True
    if value in ('False', 'false', '0'):
        return False
    raise ValueError("Can't parse boolean value: %s" % value)


class LfsPlugin(Plugin):
    def get_parser(self, fin):
        # defaults
        kwargs = {
            'locale': 'sv_SE',
            'account_id': None,
            'bank_id': 'LFS',
            'currency': 'SEK',
        }

        # overwrite from settings
        if self.settings:
            if 'locale' in self.settings:
                kwargs['locale'] = parse_bool(self.settings.get('locale'))
            if 'account_id' in self.settings:
                kwargs['account_id'] = self.settings.get('account_id')
            if 'bank_id' in self.settings:
                kwargs['bank_id'] = self.settings.get('bank_id')
            if 'currency' in self.settings:
                kwargs['currency'] = self.settings.get('currency')

        # overwrite from env
        for k, v in kwargs.items():
            k2 = 'OFX_%s' % k.upper()
            kwargs[k] = os.getenv(k2, v)

        return LfsStatementParser(fin, **kwargs)



import csv

from ofxstatement import statement
from ofxstatement.plugin import Plugin
from ofxstatement.parser import CsvStatementParser
from ofxstatement.parser import StatementParser
from ofxstatement.statement import StatementLine

class IngBePlugin(Plugin):
    """ING Belgium Plugin
    """

    def get_parser(self, filename):
        f = open(filename, 'r', encoding=self.settings.get("charset", "ISO-8859-1"))
        parser = IngBeParser(f)
        return parser

class IngBeParser(CsvStatementParser):

    date_format = "%d/%m/%Y"
    mappings = {
        'check_no': 3,
        'date': 5,
        'payee': 2,
        'memo': 8,
        'amount': 6
    }
    
    def parse(self):
        """Main entry point for parsers

        super() implementation will call to split_records and parse_record to
        process the file.
        """
        stmt = super(IngBeParser, self).parse()
        statement.recalculate_balance(stmt)
        return stmt

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        
        reader = csv.reader(self.fin, delimiter=';')
        next(reader, None)
        return reader

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        
        # Remove non CSV cr*p and zero-value notifications
        if(line[5] and not (line[6]=="0")):          
            transaction_id = line[3]
            date = line[4]
            date_value = line[5]
            if(line[2]):
                account_to = line[2]
            else:
                account_to = ' '.join(line[8].split())
		    	
            currency = line[7]
            line[6] = line[6].replace(",", ".")
            amount = line[6]

            
            # Pack info in description
            line[8] = ' '.join(line[8].split())+"-"+' '.join(line[9].split())+"-"+' '.join(line[10].split())
            description = line[8]

            stmtline = super(IngBeParser, self).parse_record(line)
            stmtline.trntype = 'DEBIT' if stmtline.amount < 0 else 'CREDIT'
        
            return stmtline



import os
import datetime
from decimal import Decimal

from ofxstatement.plugins.litas_esis import LitasEsisCsvStatementParser


def test_LitasEsisCsvStatementParser() -> None:

    # Open sample csv to parse
    csvfile = os.path.join(os.path.dirname(__file__), "samples", "litas_esis.acc")

    # Create parser object and parse:
    fin = open(csvfile, "r", encoding="cp1257")
    parser = LitasEsisCsvStatementParser(fin)
    statement = parser.parse()

    # Check what we've got:
    assert statement.account_id == "LT000000000000000000LTL"
    assert len(statement.lines) == 7
    assert statement.start_balance == Decimal("251.75")
    assert statement.start_date == datetime.datetime(2012, 1, 1, 0, 0)
    assert statement.end_balance == Decimal("74.83")
    assert statement.end_date == datetime.datetime(2012, 3, 4, 0, 0)
    assert statement.currency == "LTL"

    # Check first line:
    line = statement.lines[0]
    assert line.amount == Decimal("-1")
    assert line.payee == "AB DNB BANKAS"
    assert line.memo == "Mokestis už sąskaitos aptarnavimą"
    assert line.id == "1987555498"

    # Check credit line:
    line = statement.lines[3]
    assert line.amount == Decimal("300")
    assert line.payee == "LINUS TORVALDS"
    assert line.memo == "Hello World"
    assert line.id == "2003969289"


def test_LitasEsisCsvStatementParser_swap_payee_memo() -> None:
    # Test ability to swap payee and memo in LitasEsisCsvStatementParser

    # Open sample csv to parse
    csvfile = os.path.join(os.path.dirname(__file__), "samples", "litas_esis.acc")

    # Create parser object and parse:
    fin = open(csvfile, "r", encoding="cp1257")
    parser = LitasEsisCsvStatementParser(fin)
    parser.swap_payee_and_memo()
    statement = parser.parse()

    # Memo and payee should be swapped
    line = statement.lines[3]
    assert line.memo == "LINUS TORVALDS"
    assert line.payee == "Hello World"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from setuptools import find_packages
from distutils.core import setup

version = '1.0.5'
with open('README.rst', encoding = "utf-8") as f:
    long_description = f.read()

setup(name='ofxstatement-be-argenta',
      version=version,
      author='Wout Breugelmans',
      author_email='Wout.Breugelmans+ofxstatement@gmail.com',
      url='https://github.com/woutbr/ofxstatement-be-argenta',
      description=('ofxstatement plugin for Argenta'),
      long_description=long_description,
      long_description_content_type='text/x-rst',
      license='MIT',
      keywords=['ofx', 'ofxstatement', 'argenta'],
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'Topic :: Office/Business :: Financial :: Accounting',
          'Topic :: Utilities',
          'Environment :: Console',
          'Operating System :: OS Independent',
          'License :: OSI Approved :: MIT License',
      ],
      packages=find_packages(),
      namespace_packages=['ofxstatement', 'ofxstatement.plugins'],
      entry_points={
          'ofxstatement': ['argenta = ofxstatement.plugins.argenta:ArgentaPlugin'],
          'console_scripts': ['ofx-argenta-convert = ofxstatement_be_argenta.convert:convert']
      },
      install_requires=[
          'ofxstatement>=0.6.1',
          'openpyxl>=2.6.2',
          'click>=6.7'
      ],
      test_suite='ofxstatement.plugins.tests',
      include_package_data=True,
      zip_safe=True
      )



#    VTB (https://www.vtb.ru/) plugin for ofxstatement
#
#    Copyright 2019 Rashit Azizbaev <syndicut@gmail.com>
#    Copyright 2020 Dmitry Pavlov <zeldigas@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3 as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement import statement

import csv
from decimal import Decimal
from datetime import datetime

default_encoding = 'cp1251'
delimiter = ';'
operation_date_format = '%Y-%m-%d %H:%M:%S'

dates_skip_lines = 2
statement_info_skip_lines = 3
balance_info_skip_lines = 3

dates_fieldnames = [
    'name',
    'value',
]

statement_info_fieldnames = [
    'type',
    'account_id',
]

balance_info_fieldnames = [
    'currency',
    'end_balance',
    'income',
    'withdrawl',
    'blocked',
]

records_fieldnames = [
    'account_id',
    'operation_date',
    'processing_date',
    'operation_amount',
    'operation_currency',
    'account_amount',
    'account_currency',
    'reason',
    'status',
]

statuses = {
    'PROCESSING': 'В обработке',
}

card_info_prefix = "Карта *"
card_info_prefix_len = len(card_info_prefix)+5


class VtbStatementParser(StatementParser):

    statement = None

    def __init__(self, fin):
        super().__init__()
        self.statement = statement.Statement()
        self.fin = fin
        self.user_date = False

    def parse(self):
        """Main entry point for parsers

        super() implementation will call to split_records and parse_record to
        process the file.
        """
        dates_reader = csv.DictReader(self.fin, delimiter=delimiter, fieldnames=dates_fieldnames)
        start_date_entry = next(dates_reader)
        end_date_entry = next(dates_reader)
        self.statement.start_date = self.parse_datetime(start_date_entry['value'])
        self.statement.end_date = self.parse_datetime(end_date_entry['value'])

        self.skip_lines(dates_skip_lines)

        statement_info_reader = csv.DictReader(self.fin, delimiter=delimiter, fieldnames=statement_info_fieldnames)
        statement_info_entry = next(statement_info_reader)
        if self.statement.account_id is None:
            self.statement.account_id = self.parse_account_id(statement_info_entry['account_id'])

        self.skip_lines(statement_info_skip_lines)

        balance_info_reader = csv.DictReader(self.fin, delimiter=delimiter, fieldnames=balance_info_fieldnames)
        balance_info_item = next(balance_info_reader)
        if self.statement.currency is None:
            self.statement.currency = balance_info_item['currency']

        self.statement.end_balance = self._parse_decimal(balance_info_item['end_balance'])
        self.statement.start_balance = self.statement.end_balance - sum((self._parse_decimal(balance_info_item['income']),
                                                                         self._parse_decimal(balance_info_item['withdrawl'])))

        self.skip_lines(balance_info_skip_lines)

        super(VtbStatementParser, self).parse()
        return self.statement

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        return csv.DictReader(self.fin, delimiter=delimiter, fieldnames=records_fieldnames)

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        transaction = statement.StatementLine()

        transaction.date_user = datetime.strptime(line['operation_date'], operation_date_format)
        if line['status'] != statuses['PROCESSING']:
            if self.user_date:
                transaction.date = transaction.date_user
            else:
                transaction.date = self.parse_datetime(line['processing_date'])
        transaction.memo = line['reason']
        transaction.amount = self._parse_decimal(line['account_amount'])
        transaction.payee = self.parse_payee(line['reason'])
        transaction.trntype = self.parse_type(transaction.amount)

        # as csv file does not contain explicit id of transaction, generating artificial one
        # using operation date as main date in all cases
        transaction.id = statement.generate_transaction_id(statement.StatementLine(
            date=transaction.date_user, memo=transaction.memo, amount=transaction.amount
        ))

        return transaction

    @staticmethod
    def parse_account_id(value):
        return value.lstrip("'")

    @staticmethod
    def parse_payee(reason):
        if reason.startswith(card_info_prefix):
            return reason[card_info_prefix_len:]
        else:
            return reason

    @staticmethod
    def parse_type(amount):
        result = 'CHECK'
        if amount > 0:
            result = 'DEBIT'
        elif amount < 0:
            result = 'CREDIT'

        return result

    def skip_lines(self, lines_count):
        for _ in range(lines_count):
            next(self.fin)

    @staticmethod
    def _parse_decimal(value):
        # some plugins pass localised numbers, clean them up
        return Decimal(value.replace(",", ".").replace(" ", ""))


class VtbPlugin(Plugin):
    """VTB bank CSV (https://www.vtb.ru)
    """

    def get_parser(self, fin):
        f = open(fin, 'r', encoding=self.settings.get('file_encoding', default_encoding))
        parser = VtbStatementParser(f)
        parser.statement.currency = self.settings.get('currency')
        parser.statement.account_id = self.settings.get('account')
        parser.statement.bank_id = self.settings.get('bank', 'VTB')
        parser.user_date = self.settings.get('user_date', 'false') == 'true'
        return parser



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from distutils.core import setup

version = '0.0.1'
with open('README.rst', encoding="utf-8") as f:
    long_description = f.read()

setup(name='ofxstatement-lfs',
      version=version,
      author='Alexander Krasnukhin',
      author_email='the.malkolm@gmail.com',
      url='https://github.com/themalkolm/ofxstatement-lfs',
      description=('ofxstatement plugins for Lansforsakringar'),
      long_description=long_description,
      license='MIT',
      keywords=['ofx', 'ofxstatement', 'lfs', 'lansforsakringar'],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'Topic :: Office/Business :: Financial :: Accounting',
          'Topic :: Utilities',
          'Environment :: Console',
          'Operating System :: OS Independent'
      ],
      packages=['ofxstatement', 'ofxstatement.plugins'],
      namespace_packages=['ofxstatement', 'ofxstatement.plugins'],
      entry_points={
          'ofxstatement': ['lfs = ofxstatement.plugins.lfs:LfsPlugin']
      },
      install_requires=[
          'appdirs==1.4.3',
          'ofxstatement==0.6.1',
          'xlrd==1.1.0',
      ],
      test_suite='ofxstatement.plugins.tests',
      include_package_data=True,
      zip_safe=True
      )



import os
import unittest
import tempfile
import io
import logging
import logging.handlers
import shutil
import mock

from ofxstatement import tool, statement, configuration, parser, exceptions

class TestTestFiles(unittest.TestCase):

    def test_with_saldo(self):
        ret = tool.run(['convert', '--type=ingde', './test/test_with_saldo.csv', './test/out.ofx'])
        self.assertEqual(ret, 0)
        self.assertEqual(
            self.log.getvalue().splitlines(),
            ["INFO: Conversion completed: %s" % './test/test_with_saldo.csv'])

    def test_without_saldo(self):
        with self.assertRaises(RuntimeError):
            tool.run(['convert', '--type=ingde', './test/test_without_saldo.csv', './test/out.ofx'])

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(suffix='ofxstatement')
        self.setUpLogging()

    def tearDown(self):
        self.tearDownLogging()
        shutil.rmtree(self.tmpdir)

    def setUpLogging(self):
        self.log = io.StringIO()
        fmt = logging.Formatter("%(levelname)s: %(message)s")
        self.loghandler = logging.StreamHandler(self.log)
        self.loghandler.setFormatter(fmt)
        logging.root.addHandler(self.loghandler)
        logging.root.setLevel(logging.DEBUG)

    def tearDownLogging(self):
        logging.root.removeHandler(self.loghandler)

if __name__ == '__main__':
    unittest.main()


from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
import csv


class DABCsvStatementParser(CsvStatementParser):

    # 0 Buchungstag
    # 1 Valuta
    # 2 Buchungstext
    # 3 Kontonummer
    # 4 Auftraggeber / Empfänger
    # 5 Konto/IBAN
    # 6 BLZ/BIC
    # 7 Verwendungszweck
    # 8 Betrag in EUR

    mappings = {"date": 1, "payee": 4, "memo": 7, "amount": 8}
    date_format = "%d.%m.%Y"

    def split_records(self):
        return csv.reader(self.fin, delimiter=';')

    def parse_record(self, line):

        if self.cur_record == 2:
            self.statement.currency = line[8].strip('Betrag in ')
            return None

        if self.cur_record <= 2:
            return None

        # Remove dots (German decimal point handling)
        # 2.000,00 => 2000,00
        line[8] = line[8].replace('.', '')

        # Replace comma with dot (German decimal point handling)
        # 2000.00 => 2000.00
        line[8] = line[8].replace(',', '.')

        # fill statement line according to mappings
        sl = super(DABCsvStatementParser, self).parse_record(line)
        return sl


class DABPlugin(Plugin):
    name = "dab"

    def get_parser(self, fin):
        f = open(fin, "r", encoding='utf-8')
        parser = DABCsvStatementParser(f)
#        parser.statement.account_id = self.settings['account']
#        parser.statement.bank_id = self.settings.get('bank', 'DAB Depotkonto')
        parser.statement.account_id = "DAB Depotkonto"
        parser.statement.bank_id = "DAB"
        return parser




from ofxstatement.plugin import Plugin
from ofxstatement.parser import StatementParser
from ofxstatement.statement import StatementLine, Statement
import xml.etree.ElementTree as ET
from datetime import datetime

class ArionBankiPlugin(Plugin):
    """Arionbanki plugin"""

    def get_parser(self, filename):
        return ArionBankiParser(filename)


class ArionBankiParser(StatementParser):
    """Arionbanki parser"""
    def __init__(self, filename):
        self.statement = Statement()
        self.filename = filename
        self.tree = None

    def parse(self):
        """Main entry point for parsers

        super() implementation will call to split_records and parse_record to
        process the file.
        """
        with open(self.filename, encoding="latin-1") as arionfile:
            self.tree = ET.parse(arionfile)
        return super(ArionBankiParser, self).parse()

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        root = self.tree.getroot()
        self.statement.account_id = root[0][0].text
        return root[1:-1]

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        tags = {element.tag: element.text for element in line}
        record_id = tags['Tilvisun']
        date = datetime.strptime(tags['Dagsetning'], "%d.%m.%Y %H:%M:%S")
        memo = tags['Skyring']
        amount = float(tags['Upphaed'].replace('.', '').replace(',', '.'))
        return StatementLine(record_id, date, memo, amount)



from setuptools import setup, find_packages
import os

version = '1.3.0'

setup(name='banking.statements.nordea',
      version=version,
      description="Account statement reader plugin for Nordea of Finland",
      long_description=open("README.md").read() + "\n" +
           open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Programming Language :: Python",
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3',
        'Natural Language :: English',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Topic :: Office/Business :: Financial :: Accounting',
        'Topic :: Utilities',
        'Environment :: Console',
        'Operating System :: OS Independent',
        ],
      keywords=['ofxstatement', 'ofx'],
      author='Petri Savolainen',
      author_email='petri.savolainen@koodaamo.fi',
      url='https://github.com/koodaamo/banking.statements.nordea',
      license='GPLv3',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['banking', 'banking.statements'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'ofxstatement'
      ],
      entry_points="""
      [ofxstatement]
      nordea = banking.statements.nordea.plugin:NordeaPlugin
      """,
      )



"""Parser implementation for Citadele generated statement reports"""

import re
import logging
from xml.etree import ElementTree

from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine


class CitadeleLVStatementParser(StatementParser):
    date_format = "%Y-%m-%d"

    statement = None
    fin = None  # file input stream

    debug = (logging.getLogger().getEffectiveLevel() == logging.DEBUG)

    def __init__(self, fin):
        self.statement = Statement()
        self.fin = fin

    def split_records(self):
        xml = ElementTree.parse(self.fin)
        xml = xml.getroot()

        namespaces = {'ns': xml.tag[1:].partition("}")[0]}
        statement = xml.find('ns:Statement', namespaces=namespaces)

        period = statement.find('ns:Period', namespaces=namespaces)
        self.statement.start_date = self.parse_datetime(period.find('ns:StartDate', namespaces=namespaces).text)
        self.statement.end_date = self.parse_datetime(period.find('ns:EndDate', namespaces=namespaces).text)

        account = statement.find('ns:AccountSet', namespaces=namespaces)
        if not self.statement.account_id:
            self.statement.account_id = account.find('ns:AccNo', namespaces=namespaces).text

        transactions = account.find('ns:CcyStmt', namespaces=namespaces)
        self.statement.start_balance = self.parse_float(transactions.find('ns:OpenBal', namespaces=namespaces).text)

        all_transactions = transactions.findall('ns:TrxSet', namespaces=namespaces)

        return all_transactions

    def parse_record(self, line):
        # Namespace stuff
        namespaces = {'ns': line.tag[1:].partition("}")[0]}

        # Get all fields
        type_code = line.find('ns:TypeCode', namespaces=namespaces).text
        date = line.find('ns:BookDate', namespaces=namespaces).text
        c_or_d = line.find('ns:CorD', namespaces=namespaces).text
        amount = line.find('ns:AccAmt', namespaces=namespaces).text
        id = line.find('ns:BankRef', namespaces=namespaces).text
        note = line.find('ns:PmtInfo', namespaces=namespaces).text

        # Payee name
        payee_name = None
        payee = line.find('ns:CPartySet', namespaces=namespaces)
        if payee:
            payee_account = payee.find('ns:AccHolder', namespaces=namespaces)
            if payee_account:
                payee_name = payee_account.find('ns:Name', namespaces=namespaces).text

        # Create statement line
        stmt_line = StatementLine(id, self.parse_datetime(date), note, self.parse_float(amount))
        stmt_line.payee = payee_name

        # Credit & Debit stuff
        stmt_line.trntype = "DEP"
        if c_or_d == 'D':
            stmt_line.amount = -stmt_line.amount
            stmt_line.trntype = "DEBIT"

        # Various types
        if type_code == 'CHOU':
            stmt_line.trntype = "ATM"
        elif type_code == 'MEMD':
            stmt_line.trntype = "SRVCHG"
        elif type_code == 'OUTP':
            stmt_line.trntype = "PAYMENT"
        elif type_code == 'INP':
            stmt_line.trntype = "XFER"

        # DEBUG
        if self.debug:
            print(stmt_line, stmt_line.trntype)

        return stmt_line

    def parse_float(self, value):
        return value if isinstance(value, float) else float(value.replace(',', '.'))


class CitadeleLVPlugin(Plugin):
    """Latvian Citadele CSV"""

    def get_parser(self, fin):
        parser = CitadeleLVStatementParser(fin)
        parser.statement.currency = self.settings.get('currency', 'EUR')
        return parser



#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.

import datetime
from decimal import Decimal
import os
import unittest
from ofxstatement.statement import generate_transaction_id

from ofxstatement.plugins.easybank \
    import EasybankCreditCardCsvParser, EasybankGiroCsvParser


class TestEasybankCreditCardCsvParser(unittest.TestCase):
    """Unit tests for EasybankCreditCardCsvParser."""

    def setUp(self):
        csvfile = os.path.join(
            os.path.dirname(__file__), 'samples', 'easybank-creditcard.csv')
        with open(csvfile, 'r', encoding='cp1252') as fin:
            self.statement = EasybankCreditCardCsvParser(fin).parse()

    def test_statement_properties(self):
        self.assertEqual(len(self.statement.lines), 3)
        self.assertEqual(self.statement.start_balance, Decimal('0.0'))
        self.assertEqual(self.statement.end_balance, Decimal('2.31'))
        self.assertEqual(self.statement.currency, "EUR")
        self.assertEqual(self.statement.account_id, "12345678901")
        self.assertEqual(
            self.statement.start_date, datetime.datetime(2013, 2, 19, 0, 0))
        self.assertEqual(
            self.statement.end_date, datetime.datetime(2013, 7, 2, 0, 0))

    def test_line0_debit(self):
        line = self.statement.lines[0]
        self.assertEqual(line.amount, Decimal('-5.99'))
        self.assertEqual(line.memo, "Some vendor/info")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2013, 7, 2, 0, 0))
        self.assertEqual(line.id, "12345678909876543212345")

    def test_line1_credit(self):
        line = self.statement.lines[1]
        self.assertEqual(line.amount, Decimal('30.99'))
        self.assertEqual(line.memo, "Another vendor")
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2013, 6, 21, 0, 0))
        self.assertEqual(line.id, "23456789098765432123456")

    def test_line2_debit_with_foreign_currency(self):
        line = self.statement.lines[2]
        self.assertEqual(line.amount, Decimal('-22.69'))
        self.assertEqual(line.memo, "Someone (GBP 22,89)")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2013, 2, 19, 0, 0))
        self.assertEqual(line.id, "34567890987654321234567")


class TestEasybankGiroCsvParser(unittest.TestCase):
    """Unit tests for EasybankGiroCsvParser."""

    def setUp(self):
        csvfile = os.path.join(
            os.path.dirname(__file__), 'samples', 'easybank-giro.csv')
        with open(csvfile, 'r', encoding='cp1252') as fin:
            self.statement = EasybankGiroCsvParser(fin).parse()

    def test_statement_properties(self):
        self.assertEqual(len(self.statement.lines), 10)
        self.assertEqual(self.statement.start_balance, Decimal('0.0'))
        self.assertEqual(self.statement.end_balance, Decimal('-1562.86'))
        self.assertEqual(self.statement.currency, "EUR")
        self.assertEqual(self.statement.account_id, "AT123456789012345678")
        self.assertEqual(
            self.statement.start_date, datetime.datetime(2014, 1, 1, 0, 0))
        self.assertEqual(
            self.statement.end_date, datetime.datetime(2015, 10, 7, 0, 0))

    def test_line0_interest_paid(self):
        line = self.statement.lines[0]
        self.assertEqual(line.check_no, "1")
        self.assertEqual(line.amount, Decimal('-0.42'))
        self.assertEqual(line.memo, "Einbehaltene KESt")
        self.assertEqual(line.payee, line.memo)
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2014, 1, 1, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line1_interest_earned(self):
        line = self.statement.lines[1]
        self.assertEqual(line.check_no, "2")
        self.assertEqual(line.amount, Decimal('1.23'))
        self.assertEqual(line.memo, "Zinsen HABEN")
        self.assertEqual(line.payee, line.memo)
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2014, 1, 1, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line2_debit_with_iban_bic(self):
        line = self.statement.lines[2]
        self.assertEqual(line.check_no, "3")
        self.assertEqual(line.amount, Decimal('-123.45'))
        self.assertEqual(line.memo, "Usage, specific reason")
        self.assertEqual(
            line.payee, "Payment receiver (AT098765432109876543 ABCDEF1G235)")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2014, 1, 4, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line3_debit_legacy_bank_account(self):
        line = self.statement.lines[3]
        self.assertEqual(line.check_no, "4")
        self.assertEqual(line.amount, Decimal('-32'))
        self.assertEqual(line.memo, "Abbuchung Einzugsermächtigung")
        self.assertEqual(line.payee, "Amazon *Mktplce EU-AT (01234567890 01234)")  # noqa: E501
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2014, 1, 8, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line4_debit_iban_only(self):
        line = self.statement.lines[4]
        self.assertEqual(line.check_no, "5")
        self.assertEqual(line.amount, Decimal('-1001.00'))
        self.assertEqual(line.memo, "CustomerNo: XXXXX OrderNr: YYYYYYYY")
        self.assertEqual(line.payee, "Payment receiver (AT098765432109876543)")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2014, 1, 19, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line5_debit_withdraw(self):
        line = self.statement.lines[5]
        self.assertEqual(line.check_no, "6")
        self.assertEqual(line.amount, Decimal('-400'))
        self.assertEqual(line.memo, "Auszahlung Maestro 10.01")
        self.assertEqual(line.payee, "AUTOMAT 01234567 K1 27.07.UM 18.57")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2014, 1, 28, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line6_debit_with_more_text_before_check_no(self):
        line = self.statement.lines[6]
        self.assertEqual(line.check_no, "7")
        self.assertEqual(line.amount, Decimal('-20.9'))
        self.assertEqual(line.memo, "XYZ INVOICE 01/14 123456789012345")
        self.assertEqual(
            line.payee,
            "Foobar XZY service AG (AT098765432109876543 ABCDEF1G235)")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2014, 1, 31, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line7_debit_with_more_text_before_check_no_and_without_banking_infos(self):    # NOQA
        line = self.statement.lines[7]
        self.assertEqual(line.check_no, "8")
        self.assertEqual(line.amount, Decimal('-8.4'))
        self.assertEqual(line.memo, "AT 8,40 DEBIT POS 18.0 5.14 10.08K1")
        self.assertEqual(line.payee, "Somebody someony somewhere")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2014, 2, 21, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line8_credit_iban_bic(self):
        line = self.statement.lines[8]
        self.assertEqual(line.check_no, "9")
        self.assertEqual(line.amount, Decimal('12.1'))
        self.assertEqual(line.memo, "Gutschrift Überweisung")
        self.assertEqual(
            line.payee, "Some person (AT098765432109876543 ABCDEF1G235)")
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2014, 2, 28, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line9_memo_starts_with_forward_slash(self):
        line = self.statement.lines[9]
        self.assertEqual(line.check_no, "10")
        self.assertEqual(line.amount, Decimal('9.98'))
        self.assertEqual(line.memo, "/INV/123456790 1.10.2015")
        self.assertEqual(
            line.payee, "Some company (AT098765432109876543 ABCDEF12345)")
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2015, 10, 7, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



#!/usr/bin/python3
from setuptools import find_packages
from distutils.core import setup

version = "1.2.1.dev0"

with open("README.rst") as f:
    long_description = f.read()

setup(
    name="ofxstatement-lithuanian",
    version=version,
    author="Andrey Lebedev",
    author_email="andrey@lebedev.lt",
    url="https://github.com/kedder/ofxstatement-lithuanian",
    description=("Statement parsers for banks, operatiing in Lithuania"),
    long_description=long_description,
    license="GPLv3",
    keywords=["ofxstatement", "lithuanian"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "Natural Language :: English",
        "Topic :: Office/Business :: Financial :: Accounting",
        "Topic :: Utilities",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU Affero General Public License v3",
    ],
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["ofxstatement", "ofxstatement.plugins"],
    entry_points={
        "ofxstatement": [
            "litas-esis = ofxstatement.plugins.litas_esis:LitasEsisPlugin",
            "swedbank = ofxstatement.plugins.swedbank:SwedbankPlugin",
            "danske = ofxstatement.plugins.danske:DanskePlugin",
        ]
    },
    install_requires=["ofxstatement>=0.7.0"],
    test_suite="ofxstatement.plugins.tests",
    include_package_data=True,
    zip_safe=True,
)



import glob
import logging
import os.path
import unittest

from ofxstatement.plugins.gpc import GPCParser


class TestParser(unittest.TestCase):
    pass


def test_generator(test_file_name):
    def test_parser(self):
        with open(test_file_name, 'rb') as test_file:
            parser = GPCParser(test_file)
            logging.debug('parser = dir {}'.format(dir(parser)))
            records = parser.split_records()
            self.assertGreater(len(list(records)), 0,
                               "split input into records\nrecords = %s"
                               % records)

    def test_parseLine(self):
        with open(test_file_name, 'rb') as test_file:
            parser = GPCParser(test_file)
            logging.debug('parser = dir {}'.format(dir(parser)))
            for rec_str in parser.split_records():
                rec = parser.parse_record(rec_str)
                if rec is None:
                    self.assertIsNotNone(parser.statement.start_balance)
                else:
                    self.assertIsNotNone(rec.id, "StatementLine NOT created")

    return test_parser, test_parseLine


def additional_tests():
    suite = unittest.TestSuite()
    test_samples_dir = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "samples")
    tests = glob.glob(os.path.join(test_samples_dir, '*.gpc'))
    logging.debug('tests = {}'.format(tests))
    for t in tests:
        test_base = os.path.splitext(os.path.split(t)[1])[0]
        test_core = 'test_%s' % test_base
        tests = test_generator(t)
        for test in zip(['parser', 'parseLine'], tests):
            test_name = "{}_{}".format(test_core, test[0])
            setattr(TestParser, test_name, test[1])
            suite.addTest(TestParser(test_name))
    return suite

if __name__ == '__main__':
    unittest.main()



import csv
import re

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, recalculate_balance


class PaypalPlugin(Plugin):
    """Paypal (www.paypal.com) CSV history plugin
    """

    def get_parser(self, filename):
        encoding = self.settings.get('charset', 'cp1250')
        f = open(filename, "r", encoding=encoding)
        parser = PaypalParser(f)
        parser.statement.currency = self.settings.get('currency', 'PLN')
        parser.statement.account_id = self.settings.get('account', '')
        parser.statement.bank_id = self.settings.get('bank', 'PPAL')
        return parser


class PaypalParser(CsvStatementParser):
    mappings = {"date": 0,
                "amount": 7,
                "id": 12 }
    
    encoding = 'windows-1250'
    date_format = "%d-%m-%Y"
   
    fields = ["Data", "Godzina","Strefa czasowa", "Imię i nazwisko (nazwa)",
              "Typ", "Status", "Waluta", "Brutto", "Opłata", "Netto", 
              "Z adresu e-mail", "Na adres e-mail",
              "Numer identyfikacyjny transakcji", "Status kontrahenta",
              "Status adresu", "Nazwa przedmiotu", "Identyfikator przedmiotu",
              "Koszt wysyłki oraz koszty manipulacyjne", "Kwota ubezpieczenia",
              "Podatek od sprzedaży", "Nazwa opcji 1", "Wartość opcji 1",
              "Nazwa opcji 2", "Wartość opcji 2", "Witryna aukcji", 
              "Nazwa kupującego", "Adres URL przedmiotu", "Data zamknięcia",
              "Pomocniczy numer identyfikacyjny transakcji", "Numer faktury",
              "Numer niestandardowy", "Identyfikator potwierdzenia", "Saldo",
              "Adres wiersz 1", "Adres wiersz 2/dzielnica/osiedle", "Miejscowość",
              "Stan/prowincja/województwo/region/terytorium/prefektura/republika",
              "Kod pocztowy", "Kraj", "Numer telefonu kontaktowego"]
    
    def parse(self):
        stmt = super(PaypalParser, self).parse()
        recalculate_balance(stmt)
        return stmt
    
    def split_records(self):
        return csv.reader(self.fin, delimiter=',', quotechar='"')
    
    def parse_record(self, line):
        
        if self.cur_record == 1:
            return
        
        if line[6] != self.statement.currency:
            return
              
        sl = super(PaypalParser, self).parse_record(line)

        d = dict(zip(self.fields, line))
        p = [d[k] for k in ("Typ", "Identyfikator przedmiotu", "Nazwa przedmiotu")]
        m = [d[k] for k in ("Z adresu e-mail", "Na adres e-mail")]
        m += line[26:31] + line[33:35] + line[36:39]
        sl.payee = ' - '.join([e for e in p if e])
        sl.memo  = ' '.join([e for e in m if e])
        
        if sl.amount < 0:
            sl.trntype = "DEBIT"
        elif 0 < sl.amount:
            sl.trntype = "CREDIT"

        return sl
    
    def parse_float(self, value):
        return super(PaypalParser, self).parse_float(re.sub("[ .a-zA-Z]", "", value).replace(",", "."))


import os
import datetime
from decimal import Decimal

import pytest

from ofxstatement import exceptions
from ofxstatement.ui import UI
from ofxstatement.plugins.iso20022 import Iso20022Plugin


HERE = os.path.dirname(__file__)
SAMPLES_DIR = os.path.join(HERE, "samples")


def test_parse_simple() -> None:
    # GIVEN
    plugin = Iso20022Plugin(UI(), {})

    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, "simple.xml"))

    # WHEN
    stmt = parser.parse()

    # THEN
    assert stmt is not None

    assert stmt.account_id == "LT000000000000000000"
    assert stmt.currency == "EUR"
    assert stmt.bank_id == "AGBLLT2XXXX"
    assert stmt.end_balance == Decimal("125.52")
    assert stmt.end_date == datetime.datetime(2015, 12, 31, 0, 0)
    assert stmt.start_balance == Decimal("306.53")
    assert stmt.start_date == datetime.datetime(2015, 12, 1, 0, 0)

    assert len(stmt.lines) == 4

    assert all(l.amount for l in stmt.lines)

    line0 = stmt.lines[0]

    assert line0.amount == Decimal("-0.29")
    assert line0.memo == u"Sąskaitos aptarnavimo mokestis"
    assert line0.date == datetime.datetime(2016, 1, 1, 0, 0)
    assert line0.date_user == datetime.datetime(2015, 12, 31, 0, 0)
    assert line0.payee == u"AB DNB Bankas"
    assert line0.refnum == "FC1261858984"


def test_parse_unconfigured_currency() -> None:
    # GIVEN
    plugin = Iso20022Plugin(UI(), {})
    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, "gcamp6.xml"))

    # WHEN
    with pytest.raises(exceptions.ParseError):
        stmt = parser.parse()


def test_parse_gcamp6() -> None:
    # GIVEN
    config = {"currency": "XXX"}
    plugin = Iso20022Plugin(UI(), config)

    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, "gcamp6.xml"))

    # WHEN
    stmt = parser.parse()

    # THEN
    assert stmt is not None

    assert stmt.account_id == "CH2609000000924238861"
    assert stmt.currency is "XXX"

    assert stmt.bank_id is None
    assert stmt.end_balance == Decimal("10000.0")
    assert stmt.end_date == datetime.datetime(2017, 1, 31, 0, 0)
    assert stmt.start_balance == Decimal("0.0")
    assert stmt.start_date == datetime.datetime(2015, 12, 31, 0, 0)

    assert len(stmt.lines) == 5

    assert all(l.amount for l in stmt.lines)

    line0 = stmt.lines[0]

    assert line0.amount == Decimal("10000.0")
    assert line0.memo == "Account Transfer"
    assert line0.date == datetime.datetime(2016, 4, 23, 0, 0)
    assert line0.date_user == datetime.datetime(2016, 4, 23, 0, 0)
    assert line0.payee is None
    assert line0.refnum == "20160423000805545979476000000012"


def test_parse_davider80() -> None:
    # GIVEN
    config = {"currency": "CHF"}
    plugin = Iso20022Plugin(UI(), config)

    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, "davider80.xml"))

    # WHEN
    stmt = parser.parse()

    # THEN
    assert stmt is not None

    assert stmt.account_id == "CHxxxxxxxxxxxxxxxxxxx"
    assert stmt.currency is "CHF"

    assert stmt.bank_id == "Raiffeisen"
    assert stmt.end_balance == Decimal("5753.46")
    assert stmt.end_date == datetime.datetime(2017, 4, 11, 0, 0)
    assert stmt.start_balance == Decimal("9433.31")
    assert stmt.start_date == datetime.datetime(2017, 4, 1, 0, 0)

    assert len(stmt.lines) == 16

    assert all(l.amount for l in stmt.lines)

    line0 = stmt.lines[0]

    assert line0.amount == Decimal("-905.3")
    assert line0.memo == "Sistema di addebitamento diretto xxxxxxxxxxxxxxxxxxxxxx AG"
    assert line0.date == datetime.datetime(2017, 4, 3, 0, 0)
    assert line0.date_user == datetime.datetime(2017, 4, 1, 0, 0)
    assert line0.payee is None
    assert line0.refnum == "210564431020000000024556150000"


def test_unsupported() -> None:
    # GIVEN
    config = {"currency": "CHF"}
    plugin = Iso20022Plugin(UI(), config)

    # WHEN
    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, "unsupported.xml"))

    # THEN
    with pytest.raises(exceptions.ParseError):
        parser.parse()



#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.

import csv
import re
from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import generate_transaction_id
from ofxstatement.plugins.utils \
    import clean_multiple_whitespaces, fix_amount_string


class EasybankCsvParser(CsvStatementParser):
    """The csv parser for Easybank (base)."""

    date_format = "%d.%m.%Y"

    def split_records(self):
        """Split records using a custom dialect."""
        return csv.reader(self.fin, delimiter=";")


class EasybankCreditCardCsvParser(EasybankCsvParser):
    """The csv parser for Easybank (credit card)."""

    mappings = {
        "memo": 1,
        "id": 2,
        "date": 3,
        "amount": 5,
    }

    def parse(self):
        """Parse."""
        stmt = super(EasybankCreditCardCsvParser, self).parse()
        statement.recalculate_balance(stmt)
        return stmt

    def parse_record(self, line):
        """Parse a single record."""
        # Split the description into two parts and save it to the line list.
        parts = line[1].split('|')

        # 3 parts: Description, foreign language, transaction id
        # 2 parts: Description, transaction id
        if len(parts) == 3:
            line[1] = "{} ({})".format(parts[0], parts[1])
        else:
            line[1] = parts[0]
        line.insert(2, parts[-1])

        # Account id
        if not self.statement.account_id:
            self.statement.account_id = line[0]

        # Currency
        if not self.statement.currency:
            self.statement.currency = line[6]

        # Cleanup amount
        line[5] = fix_amount_string(line[5])
        line[1] = clean_multiple_whitespaces(line[1])

        # Create statement and fixup missing parts
        stmtline = super(EasybankCreditCardCsvParser, self).parse_record(line)
        stmtline.trntype = 'DEBIT' if stmtline.amount < 0 else 'CREDIT'

        return stmtline


class EasybankGiroCsvParser(EasybankCsvParser):
    """The csv parser for Easybank (giro)."""

    mappings = {
        "check_no": 1,
        "memo": 2,
        "payee": 3,
        "date": 4,
        "amount": 6,
    }

    reg_description = re.compile(r'[A-Z]{2}/000[0-9]{6}')
    reg_iban = re.compile(
        r'([A-Z]{6}[A-Z0-9]{2}[^\s]*)?\s?([A-Z]{2}[0-9]{10,34})\s(.*)')
    reg_legacy = re.compile(r'(.*)([0-9]{5,})\s([0-9]{6,})(.*)')

    def extract_check_no(self, description):
        '''Try to extract the statement check_no.'''
        result = ''
        mo = self.reg_description.search(description)
        if mo:
            result = str(int(mo.group(0).split('/')[1]))
        return result

    def extract_description(self, description):
        '''Cleanup description from a giro account.'''
        # extract iban/bic, account number, ...
        parts = [x.strip() for x in self.reg_description.split(description)]

        # parts: memo, transaction
        if not parts[1]:
            return parts[0], parts[0]

        # parts: memo, transaction, banking information
        else:
            # extract iban, bic and text
            iban_bic = self.reg_iban.search(parts[1])
            if iban_bic:
                # iban, bic and text
                if iban_bic.group(1):
                    result = '{0} ({1} {2})'.format(iban_bic.group(3),
                                                    iban_bic.group(2),
                                                    iban_bic.group(1))
                # iban only
                else:
                    result = '{0} ({1})'.format(
                        iban_bic.group(3), iban_bic.group(2))

                return parts[0], result

            # extract legacy banking number
            account_number = self.reg_legacy.search(parts[1])
            if account_number:
                if account_number.group(1):
                    text = account_number.group(1).strip()
                else:
                    text = account_number.group(4).strip()

                return parts[0], '{0} ({1} {2})'.format(
                    text, account_number.group(3), account_number.group(2))

            # Could not extract anything useful, return parts as is.
            return parts[0], parts[1]

    def parse(self):
        """Parse."""
        stmt = super(EasybankGiroCsvParser, self).parse()
        statement.recalculate_balance(stmt)
        return stmt

    def parse_record(self, line):
        """Parse a single record."""
        # Extract check_no/id
        description = line[1]
        del line[1]

        # Get check_no from description
        line.insert(1, self.extract_check_no(description))

        # Get memo and payee from description
        tt = self.extract_description(description)
        line.insert(2, tt[0])
        line.insert(3, tt[1])
        # line.insert(2, self.extract_description(description))

        # Account id
        if not self.statement.account_id:
            self.statement.account_id = line[0]

        # Currency
        if not self.statement.currency:
            self.statement.currency = line[7]

        # Cleanup parts
        line[6] = fix_amount_string(line[6])
        line[2] = clean_multiple_whitespaces(line[2])
        line[3] = clean_multiple_whitespaces(line[3])

        # Create statement and fixup missing parts
        stmtline = super(EasybankGiroCsvParser, self).parse_record(line)
        stmtline.trntype = 'DEBIT' if stmtline.amount < 0 else 'CREDIT'
        stmtline.id = generate_transaction_id(stmtline)

        return stmtline


class EasybankPlugin(Plugin):
    """Easybank (CSV)"""

    def determine_parser(self, fp):
        """Determine the parser to use based on the first booking line."""
        description = fp.readline().split(";")[1]
        fp.seek(0)  # reset pointer
        if '|' in description:
            return EasybankCreditCardCsvParser(fp)
        else:
            return EasybankGiroCsvParser(fp)

    def get_parser(self, filename):
        """Get a parser instance."""
        encoding = self.settings.get('charset', 'cp1252')
        f = open(filename, 'r', encoding=encoding)
        parser = self.determine_parser(f)
        parser.statement.bank_id = self.settings.get('bank', 'Easybank')
        return parser

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



from ofxstatement.plugin import Plugin
from ofxstatement.parser import StatementParser
from ofxstatement.statement import StatementLine
import csv
import decimal
import re

from ofxstatement.parser import CsvStatementParser
import csv
from io import StringIO

BANKSIGNATURES = (
"Date\tPosting details\tCredit\tDebit\tValue\tBalance"
)

CREDITSIGNATURES = (
"Date\tBilling month\tDescription\tCredit\tDebit\tTotal"
)

payeesplit = False

payeefinder = [
        re.compile("SALES.SERVICE DATED \d+.\d+.\d+ CARD NO. ?X+\d+ (.*)"),
        re.compile("E-FINANCE \d+-\d+-\d+ (.*)"),

        re.compile("GIRO INTERNATIONAL \w+ \d+.\d+ EXCHANGE RATE \d+.\d+ (.*)"),
        re.compile("GIRO FROM .+ MAILER: (.*)"),
        re.compile("GIRO FROM ACCOUNT (.*)"),
        
        # Cash withdrawals includes card no to see what card was used 
        re.compile("CASH WITHDRAWAL DATED \S+ CARD NO. (.*)"), 
        re.compile("CASH WITHDRAWAL ON \S+ CARD NO. (.*)"), 
        re.compile("WITHDRAWAL OF \d+.\d+.\d+ \S+ \S+ AT EXCHANGE RATE \S+ CARD NO. (.*)"),
        
        re.compile("DEBIT DIRECT ORDER CUSTOMER NUMBER (.*)"),
        re.compile("OTHER: (.*)"),
        re.compile("GIRO INTERNATIONAL \(SEPA\) \S+ \S+ EXCHANGE RATE \d+.\d+ (.*)"),
        re.compile("TRANSFER POSTFINANCE MOBILE FAST SERVICE : (.*)"),

        re.compile("PURCHASE/ONLINE SHOPPING OF \S+ CARD NO. (.*)"),
        
        ## The following are those that dont have a clear payee so need it all to know what is is
        re.compile("(LOAD CREDIT CARD ACCOUNT .*)"), # no unique id so leaving it as is.
        re.compile("(E-FINANCE ACCOUNT: .*)"), # keep all
        re.compile("(INTEREST STATEMENT .*)")
        
        ]

                
                
class PostFinanceBankParser(CsvStatementParser):

    mappings = { "date": 0, "memo": 1}
    date_format = "%Y%m%d"

    def split_records(self):
        #print("split records")
        return csv.reader(self.fin, delimiter='\t', quotechar='"')

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        #print(line)
        sl = StatementLine()
        sl = super(PostFinanceBankParser,self).parse_record(line)

        if not line[3]:
            sl.amount = self.parse_float(line[2])
            sl.trntype='CREDIT'
        else:
            sl.amount = self.parse_float("-" + line[3])
            sl.trntype='DEBIT'

        sl.payee = sl.memo

        if payeesplit: #TODO: enable via settings
            found = False
            for regex in payeefinder:
                m = regex.match(sl.payee)
                if m:
                    found = True
                    sl.payee = m.group(1)

                 #if not found:
                 #   print ("Found No PAYEE!" + sl.memo)
        
        return sl

class PostFinanceCreditParser(CsvStatementParser):

    mappings = { "date": 0, "memo": 2}
    date_format = "%Y%m%d"

    def split_records(self):
        result = []

        datepattern = re.compile("^\d\d\d\d\d\d\d\d\t")

        previousline = None
        
        for line in self.fin:
            line = line.rstrip()
            if(datepattern.match(line)):
                if previousline:
                    result.append(previousline)
                    previousline = None
                previousline = line
            else:
                if previousline:
                    previousline += " " + line
                else:
                    previousline = line

        if previousline:
            result.append(previousline)
            
        return csv.reader(result, delimiter='\t', quotechar='"')

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        #print(line)
        sl = StatementLine()
        sl = super(PostFinanceCreditParser,self).parse_record(line)

        #print(line)
        if not line[3]:
            sl.amount = self.parse_float(line[4])
            sl.trntype='CREDIT'
        else:
            sl.amount = self.parse_float("-" + line[3])
            sl.trntype='DEBIT'

        sl.payee = sl.memo
        
        return sl

class PostFinanceBank(Plugin):
    """PostFinance Bank .tab format
    """

    def get_csv_signature(self, csvfile):
        #print("get_csv_signature")
        return csvfile.readline().strip()
    
    def get_parser(self, fin):
        #print("get_parser")
        f = open(fin, "r", encoding="latin1")
        signature = self.get_csv_signature(f)
        if signature in BANKSIGNATURES:
            parser = PostFinanceBankParser(f)
            parser.statement.account_id = self.settings.get('account','default')
            parser.statement.currency = self.settings.get('currency', 'CHF')
            parser.statement.bank_id =  self.settings.get('bank', 'PostFinance')
            return parser
        elif signature in CREDITSIGNATURES:
            parser = PostFinanceCreditParser(f)
            parser.statement.account_id = self.settings.get('card','creditcard')
            parser.statement.currency = self.settings.get('currency', 'CHF')
            parser.statement.bank_id =  self.settings.get('bank', 'PostFinance')
            return parser
            
        
        # no plugin with matching signature was found
        f.close()
        raise Exception(signature + " does not match")



import csv
from datetime import datetime
import re

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement


class BankSMARTPlugin(Plugin):
    """Bank SMART (www.banksmart.pl) CSV history plugin
    """

    def get_parser(self, filename):
        encoding = self.settings.get('charset', 'utf8')
        f = open(filename, "r", encoding=encoding)
        parser = BankSMARTParser(f)
        parser.statement.currency = self.settings.get('currency', '')
        parser.statement.bank_id = self.settings.get('bank', 'PBPBPLPWFMB')
        parser.statement.account_id = self.settings.get('account', '')
        parser.swap_payee_and_memo = self.settings.get('swap-payee-and-memo', True)
        return parser

class BankSMARTParser(CsvStatementParser):
    mappings = {"date_user": 0,
                "date": 1,
                "payee": 2,
                "memo": 3,
                "amount": 4 }
    
    date_format = "%Y-%m-%d"
    
    def split_records(self):
        return csv.reader(self.fin, delimiter=',', quotechar='"')
    
    def parse_record(self, line):
        if self.cur_record == 1:
            return None
        if not self.statement.currency:
            self.statement.currency = line[4].split()[-1]
        if not self.statement.account_id:
            self.statement.account_id = line[6].split()[-1]
            
        sl = super(BankSMARTParser, self).parse_record(line)
        sl.date_user = datetime.strptime(sl.date_user, self.date_format)
        
        sl.id = statement.generate_transaction_id(sl)
        
        if getattr(self, 'swap_payee_and_memo', False):
            sl.memo, sl.payee = sl.payee, sl.memo

        return sl
    
    def parse_float(self, value):
        return super(BankSMARTParser, self).parse_float(re.sub("[ ,a-zA-Z]", "", value))
    


from typing import List, Optional

from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement import statement

DETAILS_FIELD = 5


class DanskeCsvStatementParser(CsvStatementParser):
    mappings = {"date": 0, "memo": 4, "amount": 8}
    date_format = "%Y:%m:%d"

    def parse(self) -> statement.Statement:
        stmt = super(DanskeCsvStatementParser, self).parse()
        statement.recalculate_balance(stmt)
        return stmt

    def parse_record(self, line: List[str]) -> Optional[statement.StatementLine]:
        if self.cur_record == 1:
            return None

        # fill statement line according to mappings
        sl = super(DanskeCsvStatementParser, self).parse_record(line)
        if sl is None:
            return None

        # generate transaction id out of available data
        sl.id = statement.generate_transaction_id(sl)
        if not sl.memo:
            sl.memo = line[DETAILS_FIELD]
        return sl

    def use_details_for_memo(self) -> None:
        self.mappings["memo"] = DETAILS_FIELD


class DanskePlugin(Plugin):
    """Lithuanian Danske bank CSV"""

    def get_parser(self, fin: str) -> DanskeCsvStatementParser:
        encoding = self.settings.get("charset", "utf-8")
        f = open(fin, "r", encoding=encoding)
        parser = DanskeCsvStatementParser(f)
        parser.statement.currency = self.settings["currency"]
        parser.statement.account_id = self.settings["account"]
        parser.statement.bank_id = self.settings.get("bank", "Danske")
        if "use-details-for-memo" in self.settings:
            parser.use_details_for_memo()
        return parser



"""Parser implementation for swedbank generated statement reports"""

import re
import csv
import logging

from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin

LINETYPE_TRANSACTION = "20"
LINETYPE_STARTBALANCE = "10"
LINETYPE_ENDBALANCE = "86"

CARD_PURCHASE_RE = re.compile(
    r"PIRKUMS \d+ (\d\d\d\d\.\d\d\.\d\d) .* \((\d+)\).*")


class SwedbankLVCsvStatementParser(CsvStatementParser):
    mappings = {"date": 2,
                "payee": 3,
                "memo": 4,
                "amount": 5,
                "id": 8}
    date_format = "%d.%m.%Y"

    debug = (logging.getLogger().getEffectiveLevel() == logging.DEBUG)

    def split_records(self):
        csv_file = csv.reader(self.fin.readlines(), delimiter=';', quotechar='"')
        return csv_file

    def parse_record(self, line):
        if self.cur_record == 1:
            # Skip header line
            return None

        # Set account_id, if its not already set
        if not self.statement.account_id:
            self.statement.account_id = line[0]

        lineType = line[1]

        if lineType == LINETYPE_TRANSACTION:

            # Fix numbers
            line[5] = line[5].replace(',', '.')

            # Convert LVL to EUR
            if line[6] == 'LVL':
                line[5] = round(float(line[5]) / 0.702804, 2)

            # parse transaction line in standard fasion
            stmtline = super(SwedbankLVCsvStatementParser, self).parse_record(line)
            stmtline.trntype = "DEP"
            if line[7] == "D":
                stmtline.amount = -stmtline.amount
                stmtline.trntype = "DEBIT"

            if line[9] == 'KOM':
                stmtline.trntype = "SRVCHG"

            m = CARD_PURCHASE_RE.match(stmtline.memo)
            if m:
                # this is an electronic purchase. extract some useful
                # information from memo field
                dt = m.group(1).replace(".", "-")
                stmtline.date_user = self.parse_datetime(dt)
                stmtline.check_no = m.group(2)

            # DEBUG
            if self.debug:
                print(stmtline, stmtline.trntype)

            return stmtline

        elif lineType == LINETYPE_ENDBALANCE:
            self.statement.end_balance = self.parse_float(line[5])
            self.statement.end_date = self.parse_datetime(line[2])

            # DEBUG
            if self.debug:
                print("End balance: %s" % self.statement.end_balance)

        elif lineType == LINETYPE_STARTBALANCE and self.statement.start_balance == None:
            self.statement.start_balance = self.parse_float(line[5])
            self.statement.start_date = self.parse_datetime(line[2])

            # DEBUG
            if self.debug:
                print("Start balance: %s" % self.statement.start_balance)

    def parse_float(self, value):
        return value if isinstance(value, float) else float(value.replace(',', '.'))

class SwedbankLVPlugin(Plugin):
    """Latvian Swedbank CSV"""

    def get_parser(self, fin):
        encoding = self.settings.get('charset', 'utf-8')
        f = open(fin, "r", encoding=encoding)
        parser = SwedbankLVCsvStatementParser(f)
        parser.statement.currency = self.settings.get('currency', 'EUR')
        return parser



# -*- encoding: utf-8 -*-
import sys
import csv
import decimal
from importlib import import_module
from io import StringIO

from ofxstatement.plugin import Plugin, PluginNotRegistered
from .parser import OPCsvStatementParser, SIGNATURES

class OPPlugin(Plugin):
    "Suomen Osuuspankki / Finnish Osuuspankki"

    def get_parser(self, fin):
        f = open(fin, "r", encoding='iso-8859-1')
        signature = f.readline().strip()
        f.seek(0)
        if signature in SIGNATURES:
            parser = OPCsvStatementParser(f)
            parser.statement.account_id = self.settings['account']
            parser.statement.currency = self.settings['currency']
            parser.statement.bank_id = self.settings.get('bank', 'Osuuspankki')
            return parser

        # no plugin with matching signature was found
        raise Exception("No suitable Osuuspankki parser found for this statement file.")



import csv
from datetime import datetime
import re

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement


class RaiffeisenPolbankPlugin(Plugin):
    """Raiffeisen-Polbank (www.raiffeisenpolbank.com) CSV history plugin
    """

    def get_parser(self, filename):
        encoding = self.settings.get('charset', 'cp1250')
        f = open(filename, "r", encoding=encoding)
        parser = RaiffeisenPolbankParser(f)
        parser.statement.bank_id = self.settings.get('bank', 'RCBWPLPW')
        parser.statement.account_id = self.settings.get('account', '')
        parser.statement.currency = self.settings.get('currency', 'PLN')
        parser.swap_payee_and_memo = self.settings.get('swap-payee-and-memo', True)
        return parser


class RaiffeisenPolbankParser(CsvStatementParser):
    mappings = {"date_user":0,
                "date": 1,
                "memo": 2,
                "payee": 3,
                "amount": 5, }
    
    date_format = "%d-%m-%Y"
    
    def parse(self):
        self.parsing_header = True
        self.last_line = None
        return super(RaiffeisenPolbankParser, self).parse()
    
    def split_records(self):
        return csv.reader(self.fin, delimiter=';')
    
    def parse_record(self, line):
        if self.cur_record == 1:
            return None
        
        if not getattr(self.statement, 'currency', None):
            self.statement.currency = line[6]
        
        sl = super(RaiffeisenPolbankParser, self).parse_record(line)
        sl.date_user = datetime.strptime(sl.date_user, self.date_format)

        if line[4]:
            sl.memo += " - " + line[4]  

        sl.id = statement.generate_transaction_id(sl)
        
        if line[2].startswith("Polecenie przelewu"):
            sl.trntype = "XFER"
        elif line[2].startswith("Podatek"):
            sl.trntype = "FEE"
        elif line[2].startswith("Opłata"):
            sl.trntype = "SRVCHG"
        elif line[2].startswith("Odsetki"):
            sl.trntype = "INT"
        else:
            sl.trntype = "XFER"

        if getattr(self, 'swap_payee_and_memo', True):
            sl.memo, sl.payee = sl.payee, sl.memo
            
        return sl
    
    def parse_float(self, value):
        return super(RaiffeisenPolbankParser, self).parse_float(re.sub("[ .a-zA-Z]", "", value).replace(",", "."))


#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.

import csv
from ofxstatement.plugin import Plugin
from ofxstatement.parser import CsvStatementParser
from ofxstatement.statement import generate_transaction_id
from ofxstatement import statement
from ofxstatement.plugins.utils import fix_amount_string


class IngDiBaCsvParser(CsvStatementParser):
    """The csv parser for ING-DiBa."""

    date_format = "%d.%m.%Y"

    mappings = {
        "memo": 1,
        "date": 2,
        "amount": 4,
        }

    def parse(self):
        """Parse."""
        stmt = super(IngDiBaCsvParser, self).parse()
        statement.recalculate_balance(stmt)
        return stmt

    def split_records(self):
        """Split records using a custom dialect."""
        return csv.reader(self.fin, delimiter=";")

    def parse_record(self, line):
        """Parse a single record."""
        # Skip header line
        if self.cur_record == 1:
            return None

        # Account id
        if not self.statement.account_id:
            self.statement.account_id = line[0]

        # Currency
        if not self.statement.currency:
            self.statement.currency = line[3]

        # Save credit/debit on line[4]
        if line[4] == "0,00":
            line[4] = fix_amount_string(line[5])
        else:
            line[4] = "-{}".format(fix_amount_string(line[4]))

        # Create statement and fixup missing parts
        stmtline = super(IngDiBaCsvParser, self).parse_record(line)
        stmtline.trntype = 'DEBIT' if stmtline.amount < 0 else 'CREDIT'
        stmtline.id = generate_transaction_id(stmtline)

        return stmtline


class IngDiBaPlugin(Plugin):
    """ING-DiBa (CSV)"""

    def get_parser(self, filename):
        """Get a parser instance."""
        encoding = self.settings.get('charset', 'iso-8859-1')
        f = open(filename, 'r', encoding=encoding)
        parser = IngDiBaCsvParser(f)
        parser.statement.bank_id = self.settings.get('bank', 'ING-DiBa')
        return parser

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



from ofxstatement.plugin import Plugin
from ofxstatement.parser import CsvStatementParser
from ofxstatement.exceptions import ParseError
import csv

LINELENGTH = 18
HEADER_START = "Rekeningnummer"


class KbcBePlugin(Plugin):
    """Belgian KBC Bank plugin for ofxstatement
    """

    def get_parser(self, filename):
        f = open(filename, 'r')
        parser = KbcBeParser(f)
        return parser


class KbcBeParser(CsvStatementParser):
    date_format = "%d/%m/%Y"

    mappings = {
        'memo': 6,
        'date': 5,
        'amount': 8,
        'check_no': 4,
        'refnum': 4
    }

    line_nr = 0

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        return csv.reader(self.fin, delimiter=';')

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        self.line_nr += 1
        if line[0] == HEADER_START:
            return None
        elif len(line) != LINELENGTH:
            raise ParseError(self.line_nr,
                             'Wrong number of fields in line! ' +
                             'Found ' + str(len(line)) + ' fields ' +
                             'but should be ' + str(LINELENGTH) + '!')

        # Check the account id. Each line should be for the same account!
        if self.statement.account_id:
            if line[0] != self.statement.account_id:
                raise ParseError(self.line_nr,
                                 'AccountID does not match on all lines! ' +
                                 'Line has ' + line[0] + ' but file ' +
                                 'started with ' + self.statement.account_id)
        else:
            self.statement.account_id = line[0]

        # Check the currency. Each line should be for the same currency!
        if self.statement.currency:
            if line[3] != self.statement.currency:
                raise ParseError(self.line_nr,
                                 'Currency does not match on all lines! ' +
                                 'Line has ' + line[3] + ' but file ' +
                                 'started with ' + self.statement.currency)
        else:
            self.statement.currency = line[3]

        # Get reconciliation data. First line is the most recent transaction
        # and contains the end balance. Last line is the oldest transaction and
        # contains the start balance plus or minus the oldest transaction
        # amount. Not perfect since we need to calculate, but still valuable.
        if self.line_nr == 2:  # First line with data is line nr 2 :-)
            # Store end balance
            self.statement.end_balance = self.parse_float(line[9])
        else:
            # Calculate start balance & store start balance,
            # will overwrite every time, last line survives.
            self.statement.start_balance = self.parse_float(line[9]) - \
                                           self.parse_float(line[8])

        stmt_ln = super(KbcBeParser, self).parse_record(line)

        return stmt_ln



from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement import statement
import csv
import re


class FlatexCsvStatementParser(CsvStatementParser):
    mappings = {"date": 0,"memo": 4, "amount": 5}
    date_format = "%d.%m.%Y"

    def split_records(self):
        return csv.reader(self.fin, delimiter=';')
 
    def parse_record(self, line):
        #Lines with no information
        if self.cur_record <= 1:
            return None
        
        if self.statement.currency==None:
            self.statement.currency=line[6]

        #Change decimalsign from , to .
        line[5]=line[5].replace('.','').replace(',','.')
            
        # fill statement line according to mappings
        sl = super(FlatexCsvStatementParser, self).parse_record(line)
        return sl    


class FlatexPlugin(Plugin):
    name = "flatex"
    def get_parser(self, fin):
        f = open(fin, "r",encoding='iso-8859-1')
        parser=FlatexCsvStatementParser(f)
        parser.statement.account_id = self.settings['account']
        parser.statement.currency = None
        parser.statement.bank_id = self.settings.get('bank', 'FLATEX_Cash') 
        return parser




import datetime
from decimal import Decimal
from unittest import mock

from ofxstatement.plugins.alfabank import AlfabankPlugin
from .util import file_sample


def test_parser():
    plugin = AlfabankPlugin(mock.Mock(), {'user_date': 'false'})
    statement = plugin.get_parser(file_sample("alfabank.csv")).parse()
    assert statement.bank_id == 'Alfabank'
    assert statement.currency == 'RUR'
    assert statement.account_id == '11111111111111111111'

    _check_line(statement.lines[0], '33123.56', datetime.datetime(2017, 5, 31, 0, 0),
                '{VO11111} Перечисление ден. средств (зарплата за май 2017 г.)', 'ABCDEF11111111111', 'DEBIT')
    _check_line(statement.lines[1], '-50', datetime.datetime(2017, 5, 3, 0, 0),
                'Комиссия за переводы в рублях за период с1МАЙ17 по 1МАЙ17 поруч.поданы ч/з '
                'ИнтернетБанк Согл.тариф.Банка Иванов Иван Иванович', 'ABCDEF22222222222', 'SRVCHG')
    _check_line(statement.lines[2], '-1500', datetime.datetime(2017, 5, 3, 0, 0),
                '123456++++++6789    11111111\\123\\Visa Direct\\SOME BANK             '
                '03.05.17 01.05.17    1500.00  RUR MCC1234', 'ABC_1AA11A', 'CREDIT')


def test_user_date():
    plugin = AlfabankPlugin(mock.Mock(), {})
    statement = plugin.get_parser(file_sample("alfabank.csv")).parse()

    assert statement.lines[2].date == datetime.datetime(2017, 5, 1, 0, 0)


def _check_line(line, amount, date, memo, refnum, trntype):
    assert line.amount == Decimal(amount)
    assert line.date == date
    assert line.memo == memo
    assert line.refnum == refnum
    assert line.trntype == trntype



import os.path

SAMPLES_DIR = os.path.join(os.path.dirname(__file__), 'samples')


def file_sample(name):
    result = os.path.join(SAMPLES_DIR, name)
    if not os.path.exists(result):
        raise RuntimeError("Failed to find " + result)
    return result



import os
import re
import datetime
from decimal import Decimal

from ofxstatement.plugins.swedbank import SwedbankCsvStatementParser


def test_SwedbankCsvStatementParser() -> None:
    # Open sample csv to parse
    csvfile = os.path.join(os.path.dirname(__file__), "samples", "swedbank.csv")

    fin = open(csvfile, "r", encoding="utf-8")
    parser = SwedbankCsvStatementParser(fin)
    parser.statement.currency = "LTL"
    statement = parser.parse()

    # Check what we've got:
    assert statement.account_id == "LT797300010XXXXXXXXX-LTL"
    assert len(statement.lines) == 5
    assert statement.start_balance == Decimal("2123.82")
    assert statement.start_date == datetime.datetime(2012, 1, 1, 0, 0)
    assert statement.end_balance == Decimal("3917.30")
    assert statement.end_date == datetime.datetime(2012, 1, 31, 0, 0)
    assert statement.currency == "LTL"

    # Check first line
    line = statement.lines[0]
    assert line.amount == Decimal("-14.34")
    assert line.payee == "McDonald's restoranas AKR Vilnius"
    assert line.memo is not None
    assert re.match(r"PIRKINYS .* 00000", line.memo)
    assert line.id == "2012010200041787"
    assert line.check_no == "059553"
    assert line.date == datetime.datetime(2012, 1, 2, 0, 0)
    assert line.date_user == datetime.datetime(2011, 12, 30, 0, 0)

    # Check line with awkward quotation marks:
    line = statement.lines[2]
    assert line.id == "2012012600096815"
    assert line.amount == Decimal("-12.20")
    assert line.payee == 'UAB "Naktida"'
    assert line.memo is not None
    assert re.match(r'PIRKINYS 0000000000000000 .* UAB "Naktida" .* 00000', line.memo)

    # Check income line:
    line = statement.lines[3]
    assert line.id == "2012011000673562"
    assert line.amount == Decimal("1600.00")
    assert line.payee == "Company"
    assert line.memo == "Salary"

    # Check line with empty payee:
    line = statement.lines[4]
    assert line.id == "2012022900875660"
    assert line.payee == ""



import doctest

from ofxstatement.plugins.lbbamazon import LbbAmazonCsvStatementParser

def doctest_LbbAmazonCsvStatementParser():
    """Test LbbAmazonCsvStatementParser

    Open sample csv to parse
        >>> import os
        >>> csvfile = os.path.join(os.path.dirname(__file__),
        ...                        'samples', 'lbbamazon.csv')

    Create parser object and parse:
        >>> fin = open(csvfile, 'r', encoding='iso-8859-1')
        >>> parser = LbbAmazonCsvStatementParser(fin)
        >>> statement = parser.parse()

    Check what we've got:
        >>> statement.account_id
        >>> len(statement.lines)
        7
        >>> statement.start_balance
        >>> statement.start_date
        >>> statement.end_balance
        >>> statement.end_date
        >>> statement.currency

    Check first line
        >>> l = statement.lines[0]
        >>> l.amount
        Decimal('-0.17')
        >>> l.payee 
        >>> l.memo
        'ABGELTUNGSSTEUER'
        >>> l.date
        datetime.datetime(2012, 12, 28, 0, 0)

    Check one more line:
        >>> l=statement.lines[2]
        >>> l.amount
        Decimal('0.75')
        >>> l.payee
        >>> l.memo
        'HANDYRABATT'
        >>> l.date
        datetime.datetime(2013, 1, 21, 0, 0)

    Check one more line with slashes in memo:
        >>> l=statement.lines[4]
        >>> l.amount
        Decimal('-30.00')
        >>> l.memo
        'AMAZON.ES COMPRA / amazon.es/ay'
        >>> l.date
        datetime.datetime(2013, 1, 7, 0, 0)

    Check one more line with amazon points but without amount:
        >>> l=statement.lines[5]
        >>> l.amount
        Decimal('0.00')
        >>> l.memo
        '+ 15.0 AMAZON.DE PUNKTE'
        >>> l.date
        datetime.datetime(2013, 1, 7, 0, 0)
              
    """

def test_suite(*args):
    return doctest.DocTestSuite(optionflags=(doctest.NORMALIZE_WHITESPACE|
                                             doctest.ELLIPSIS|
                                             doctest.REPORT_ONLY_FIRST_FAILURE|
                                             doctest.REPORT_NDIFF
                                             ))
load_tests = test_suite

#if __name__ == "__main__":
#    doctest.testmod()




"""Parser implementation for swedbank generated statement reports"""
from typing import List, Optional, Iterable
import re

from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import StatementLine

LINETYPE_TRANSACTION = "20"
LINETYPE_STARTBALANCE = "10"
LINETYPE_ENDBALANCE = "86"

CARD_PURCHASE_RE = re.compile(r"PIRKINYS \d+ (\d\d\d\d\.\d\d\.\d\d) .* \((\d+)\).*")


class SwedbankCsvStatementParser(CsvStatementParser):
    mappings = {"date": 2, "payee": 3, "memo": 4, "amount": 5, "id": 8}

    def split_records(self) -> Iterable[List[str]]:
        # We cannot parse swedbank csv as regular csv because swedbanks format
        # uses unescaped quote symbols.
        for line in self.fin.readlines():
            if not line.strip():
                yield []
                continue
            parts = [p[1:] for p in line.split('",')]
            yield parts

    def parse_record(self, line: List[str]) -> Optional[StatementLine]:
        if not line:
            return None

        if self.cur_record == 1:
            # Skip header line
            return None

        linecur = line[6]
        if self.statement.currency != linecur:
            # Skip lines with different currency
            return None

        # Split line to the parts and strip quotes around fields
        # parts = [l[1:] for l in line.split('",')]
        if not self.statement.account_id:
            self.statement.account_id = "%s-%s" % (line[0], self.statement.currency)

        lineType = line[1]

        if lineType == LINETYPE_TRANSACTION:
            # parse transaction line in standard fasion
            stmtline = super().parse_record(line)
            if stmtline is None:
                return None
            if line[7] == "D":
                assert stmtline.amount is not None
                stmtline.amount = -stmtline.amount

            if stmtline.memo:
                m = CARD_PURCHASE_RE.match(stmtline.memo)
                if m:
                    # this is an electronic purchase. extract some useful
                    # information from memo field
                    dt = m.group(1).replace(".", "-")
                    stmtline.date_user = self.parse_datetime(dt)
                    stmtline.check_no = m.group(2)

            return stmtline

        elif lineType == LINETYPE_ENDBALANCE:
            self.statement.end_balance = self.parse_float(line[5])
            self.statement.end_date = self.parse_datetime(line[2])

        elif lineType == LINETYPE_STARTBALANCE:
            self.statement.start_balance = self.parse_float(line[5])
            self.statement.start_date = self.parse_datetime(line[2])

        return None


class SwedbankPlugin(Plugin):
    """Lithuanian Swedbank CSV"""

    def get_parser(self, fin: str) -> SwedbankCsvStatementParser:
        f = open(fin, "r")
        parser = SwedbankCsvStatementParser(f)
        parser.statement.currency = self.settings.get("currency", "EUR")
        return parser



import csv
from datetime import datetime
import re

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement


class MBankPLPlugin(Plugin):
    """Polish mBank (www.mbank.pl) CSV history plugin
    """

    def get_parser(self, filename):
        encoding = self.settings.get('charset', 'cp1250')
        f = open(filename, "r", encoding=encoding)
        parser = MBankPLParser(f)
        parser.statement.bank_id = self.settings.get('bank', 'BREXPLPWMUL')
        return parser


class MBankPLParser(CsvStatementParser):
    mappings = {"date_user":0,
                "date": 1,
                "amount": 6, }
    
    date_format = "%Y-%m-%d"
    
    def parse(self):
        self.parsing_header = True
        self.last_line = None
        return super(MBankPLParser, self).parse()
    
    def split_records(self):
        return csv.reader(self.fin, delimiter=';', quotechar='"')
    
    def parse_record(self, line):
        
        if self.parsing_header:
            return self.parse_header(line)
        
        if len(line) != 9:
            return None
        
        # footer
        if line[6] == "#Saldo końcowe":
            self.statement.end_balance = self.parse_float(line[7])
            return None
        
        sl = super(MBankPLParser, self).parse_record(line)
        sl.date_user = datetime.strptime(sl.date_user, self.date_format)

        # account number and name
        sl.memo = re.sub("\s+", " ", ' '.join([line[5].strip("'"), line[4]]))
        
        # type - description
        sl.payee = re.sub("\s+", " ", ' - '.join(line[2:4]))

        # generate transaction id out of available data
        sl.id = statement.generate_transaction_id(sl)
        
        if line[2].startswith("PRZ"):
            sl.trntype = "XFER"
        elif line[2].startswith("WYPŁATA"):
            sl.trntype = "ATM"
        elif line[2].startswith("ZAKUP"):
            sl.trntype = "DEBIT"
        elif line[2].startswith("PODATEK"):
            sl.trntype = "FEE"
        elif line[2].startswith("OPŁATA"):
            sl.trntype = "SRVCHG"
        elif line[2].startswith("KAPITALIZACJA"):
            sl.trntype = "INT"

        return sl
    
    def parse_float(self, value):
        return super(MBankPLParser, self).parse_float(re.sub("[ .a-zA-Z]", "", value).replace(",", "."))
    
    def parse_header(self, line):
        
        stmt = self.statement
        last = self.last_line
        
        if not line:
            pass
        elif line[0] == "Łącznie":
            stmt.total_amount = self.parse_float(line[2])
        elif line[0] == "#Saldo początkowe":
            stmt.start_balance = self.parse_float(line[1])
        elif line[0] == "#Data operacji":
            stmt.end_balance = stmt.start_balance + stmt.total_amount
            self.parsing_header = False
        elif not last:
            pass
        elif last[0] == "#Za okres:":
            stmt.start_date = datetime.strptime(line[0], "%d.%m.%Y")
            stmt.end_date = datetime.strptime(line[1], "%d.%m.%Y")
        elif last[0] == "#Waluta":
            stmt.currency = line[0].strip()
        elif last[0] == "#Numer rachunku":
            stmt.account_id = "PL" + line[0].replace(" ", "")

        self.last_line = line
        
        return None


import xml.etree.ElementTree as ET
import datetime
import html
import re

from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine


CD_CREDIT = 'CRDT'
CD_DEBIT = 'DBIT'

class OtpPlugin(Plugin):
    """OTP (XML)
    """

    def get_parser(self, filename):
        return OtpParser(filename)


class OtpParser(object):
    def __init__(self, filename):
        self.filename = filename

    def parse(self):
        """Main entry point for parsers
        """
        self.statement = Statement()
        tree = ET.parse(self.filename)

        self._parse_statement_properties(tree)
        self._parse_lines(tree)

        return self.statement

    def _parse_statement_properties(self, tree):
        stmt = tree.find('./Rpt')

        bnk = stmt.find('./Acct/Svcr/FinInstnId/Nm')
        iban = stmt.find('./Acct/Id/Othr/Id')
        bals = stmt.findall('./Bal')

        bal_amts = {}
        bal_dates = {}
        for bal in bals:
            cd = bal.find('./Tp/CdOrPrtry/Cd')
            amt = bal.find('./Amt')
            dt = bal.find('./Dt')

            # Amount currency should match with statement currency
            bal_amts[cd.text] = self._parse_amount(amt)
            bal_dates[cd.text] = self._parse_date(dt)

        self.statement.bank_id = bnk.text
        self.statement.account_id = iban.text
        self.statement.start_balance = bal_amts['OPBD']
        self.statement.start_date = bal_dates['OPBD']
        self.statement.end_balance = bal_amts.get('CLBD', None)
        self.statement.end_date = bal_dates.get('CLBD', None)

    def _parse_lines(self, tree):
        for ntry in _findall(tree, 'Rpt/Ntry'):
            sline = self._parse_line(ntry)
            if sline is not None:
                self.statement.lines.append(sline)

    def _parse_line(self, ntry):
        # don't try to parse pending entries, too many items missing
        if _find(ntry, 'Sts').text == 'PDNG':
            return None

        sline = StatementLine()

        crdeb = _find(ntry, 'CdtDbtInd').text

        amtnode = _find(ntry, 'Amt')
        amt = self._parse_amount(amtnode)
        if crdeb == CD_DEBIT:
            amt = -amt
            payee = _find(ntry, 'NtryDtls/TxDtls/RltdPties/Cdtr/Nm')
        else:
            payee = _find(ntry, 'NtryDtls/TxDtls/RltdPties/Dbtr/Nm')
        if payee is not None:
            payee = payee.text

        sline.payee = payee
        sline.amount = amt

        dt = _find(ntry, 'ValDt')
        sline.date = self._parse_date(dt)

        bookdt = _find(ntry, 'BookgDt')
        sline.date_user = self._parse_date(bookdt)

        svcref = _find(ntry, 'NtryDtls/TxDtls/Refs/AcctSvcrRef')
        sline.refnum = getattr(svcref, 'text', None)

        rmtinf = _find(ntry, 'NtryDtls/TxDtls/RmtInf/Ustrd')
        sline.memo = rmtinf.text if rmtinf.text else ''

        addtlinf_node = _find(ntry, 'NtryDtls/TxDtls/AddtlTxInf')
        if addtlinf_node is not None and addtlinf_node.text is not None:
            addtlinf = self._parse_addtlinf(addtlinf_node)
        else:
            addtlinf = ''

        if 'VÁSÁRLÁS KÁRTYÁVAL' == addtlinf and not sline.payee:
            sline.payee = _trim_payee(sline.memo)

        sline.memo += ' ' + addtlinf

        return sline

    def _parse_date(self, dtnode):
        if dtnode is None:
            return None

        dt = _find(dtnode, 'Dt')
        dttm = _find(dtnode, 'DtTm')

        if dt is not None and dt.text is not None:
            return datetime.datetime.strptime(dt.text, "%Y-%m-%d")
        else:
            assert dttm is not None
            return datetime.datetime.fromisoformat(dttm.text, "%Y-%m-%dT%H:%M:%S")


    def _parse_amount(self, amtnode):
        if amtnode.text is not None:
            return float(amtnode.text)
        else:
            return None

    def _parse_addtlinf(self, addtlinf):
        string = '<root>' + html.unescape(addtlinf.text).replace("&", "&amp;") + '</root>'
        for infonode_label in ['narr', 'inf']:
            if ET.fromstring(string).find(infonode_label) is not None:
                return ET.fromstring(string).find(infonode_label).text
        return None


def _toxpath(spath):
    tags = spath.split('/')
    path = ['%s' % t for t in tags]
    xpath = './%s' % '/'.join(path)
    return xpath


def _find(tree, spath):
    return tree.find(_toxpath(spath))


def _findall(tree, spath):
    return tree.findall(_toxpath(spath))

_trim_payee_regexes = [
    re.compile('   .*'),
    re.compile('\d+\.\d+\.\d+ \d+'),
    re.compile('[\d,]+EUR.*')
]
def _trim_payee(payee):
    for r in _trim_payee_regexes:
        payee = r.sub('', payee)
    return payee



# -*- coding: utf-8 -*-

"""
    Author: Michal Zimen mzimen@epitheton.com
"""

from ofxstatement.plugin import Plugin
from ofxstatement.parser import StatementParser, CsvStatementParser
from ofxstatement.statement import StatementLine, Statement, BankAccount

import csv, re, datetime
from datetime import timezone, timedelta

class MBankSKPlugin(Plugin):
    """MBank.SK plugin (for developers only)
    """

    def get_parser(self, filename):
        return MBankSKParser(filename)


class MBankSKParser(CsvStatementParser):
    """
    Parsing CSV file
    """
    mappings = {"date_user":0, "date": 1,
                "payee":4, #"id": 2,
                "memo": 3, "amount": 9, }
    encoding = 'windows-1250' # 'utf-8'
    date_format = "%d-%m-%Y"

    def __init__(self, filename):
        self.filename = filename
        self.statement = Statement()
        self.last_line = ""

    def parse(self):
        """Main entry point for parsers

        super() implementation will call to split_records and parse_record to
        process the file.
        """
        with open(self.filename, "r", encoding=self.encoding) as self.csvfile:
            return super(MBankSKParser, self).parse()

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        return csv.reader(self.csvfile, delimiter=';', quotechar='"')

    def parse_float(self, value):
        value = value.replace(" ", "")
        value = value.replace(",", ".")
        return float(value)

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        if not self.statement.currency \
           and re.match(r"^#Mena ..tu", self.last_line):
            self.statement.currency = line[0]
        if not self.statement.bank_id \
           and re.match(r"^#BIC", self.last_line):
            self.statement.bank_id = line[0]
        if not self.statement.account_id \
           and re.match(r"^#..slo ..tu:", self.last_line):
            self.statement.account_id = line[0]
        if not self.statement.start_date \
           and re.match(r"^#Za obdobie:", self.last_line):
            self.statement.start_date = datetime.datetime.strptime(line[0], "%d.%m.%Y").replace(tzinfo=timezone(timedelta(hours=1)))
            self.statement.end_date = datetime.datetime.strptime(line[1], "%d.%m.%Y").replace(hour=23, minute=59, second=59, tzinfo=timezone(timedelta(hours=1)))
        if not self.statement.start_balance \
           and len(line) > 6 \
           and re.match(r"^#Po.iato.n. zostatok:", line[6]):
            self.statement.start_balance = self.parse_float(re.sub("[ .a-zA-Z]", "", line[7]).replace(",", "."))
        if not self.statement.end_balance \
           and len(line) > 6 \
           and re.match(r"^#Kone.n. zostatok:", line[6]):
            self.statement.end_balance = self.parse_float(re.sub("[ .a-zA-Z]", "", line[7]).replace(",", "."))
        self.last_line = line[0]
        if len(line) > 10:
            md1 = re.match(r"^\d{2}-\d{2}-\d{4}$", line[0])
            md2 = re.match(r"^\d{2}-\d{2}-\d{4}$", line[1])
            if md1 and md2:
                mx_date = re.search(r'DÁTUM VYKONANIA TRANSAKCIE: (\d+)-(\d+)-(\d+)', line[3])
                if mx_date:
                    line[0] = "-".join([mx_date.group(3), mx_date.group(2),
                                       mx_date.group(1)])
                # let super CSV parserd with mappings do this job instead
                stmt_line = super(MBankSKParser, self).parse_record(line)

                stmt_line.refnum = ""
                if len(line[7]):
                    stmt_line.refnum = stmt_line.refnum + "/VS" + line[7]
                if len(line[8]):
                    stmt_line.refnum = stmt_line.refnum + "/SS" + line[8]
                if len(line[6]):
                    stmt_line.refnum = stmt_line.refnum + "/KS" + line[6]
                if len(line[5]):
                    stmt_line.bank_account_to = BankAccount("", re.sub("[ ']", "", line[5]))
                else:
                    stmt_line.bank_account_to = None

                if len(stmt_line.date_user):
                    stmt_line.date_user = datetime.datetime.strptime(line[0], self.date_format).replace(tzinfo=timezone(timedelta(hours=1)))
                stmt_line.date = stmt_line.date.replace(tzinfo=timezone(timedelta(hours=1)))
                if line[2] == "PLATBA KARTOU":
                    stmt_line.trn_type = "PAYMENT"
                    payee = line[3].split('/')
                    if len(payee) > 0:
                        stmt_line.payee = payee[0].strip()
                elif line[2] == u"VÝBER V BANKOMATE":
                    stmt_line.trn_type = "ATM"
                elif line[2] == u"INKASO":
                    stmt_line.trn_type = "DIRECTDEBIT"
                else:
                    stmt_line.trn_type = "XFER"
                return stmt_line



import unittest
import glob
import os.path
from ofxstatement.plugins.maxibps import PSTextFormatParser


class TestParser(unittest.TestCase):
    pass


def test_generator(test_file_name):
    def test_parser(self):
        with open(test_file_name, "U", encoding="utf-8-sig") as test_file:
            parser = PSTextFormatParser(test_file)
            records = parser.split_records()
            self.assertGreater(len(records), 0,
                               "split input into records\nrecords = %s"
                               % records)

    def test_parseLine(self):
        with open(test_file_name, "U", encoding="utf-8-sig") as test_file:
            parser = PSTextFormatParser(test_file)
            for rec_str in parser.split_records():
                rec = parser.parse_record(rec_str)
                self.assertIsNotNone(rec, "StatementLine created")

    return test_parser, test_parseLine


def additional_tests():
    suite = unittest.TestSuite()
    test_samples_dir = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "samples")
    tests = glob.glob(os.path.join(test_samples_dir, '*.txt'))
    for t in tests:
        test_base = os.path.splitext(os.path.split(t)[1])[0]
        test_core = 'test_%s' % test_base
        tests = test_generator(t)
        for test in zip(['parser', 'parseLine'], tests):
            test_name = "{}_{}".format(test_core, test[0])
            setattr(TestParser, test_name, test[1])
            suite.addTest(TestParser(test_name))
    return suite

if __name__ == '__main__':
    unittest.main()



import csv
from datetime import datetime
import re

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
#from ofxstatement.statement import Statement


class RaiffeisenCZPlugin(Plugin):
    """Raiffeisenbank, a.s. (Czech Republic) (CSV, cp1250)
    """

    def get_parser(self, filename):
        # Open input file and set some defaults
        RaiffeisenCZPlugin.encoding = self.settings.get('charset', 'cp1250')
        f = open(filename, "r", encoding=RaiffeisenCZPlugin.encoding)
        parser = RaiffeisenCZParser(f)
        parser.statement.currency = self.settings.get('currency', 'CZK')
        parser.statement.bank_id = self.settings.get('bank', 'RZBCCZPP')
        parser.statement.account_id = self.settings.get('account', '')
        parser.statement.account_type = self.settings.get('account_type', 'CHECKING')
        parser.statement.trntype = "OTHER"
        return parser


class RaiffeisenCZParser(CsvStatementParser):

    # GnuCash recognises the following descriptive fields:
    # - Header for Transaction Journal:
    #   - Description
    #   - Notes
    # - Line item memo for each account in a single Transaction Journal:
    #   - Memo
    #
    # .payee is assigned to "Description" in GnuCash
    # .memo is assigned to "Memo" in GnuCash and also concatenated
    #       to "Notes" after "OFX ext. info" and "Trans type"
    #
    # When .payee is empty, GnuCash assigns .memo to:
    # - "Description" and does not concatenate to "Notes"
    # - "Memo"
    #
    # Although ofxstatement can create bank_account_to, GnuCash ignores it.
    #
    # In GnuCash, .check_no (if empty, then .refnum) is assigned to "Num".
    #
    # The approach is:
    # - merge counterparty name (.payee) + account number + bank code
    # - merge pmt reference (.memo) + other payment specifics (VS, KS, SS)

    date_format = "%d.%m.%Y %H:%M"
    date_format_user = "%d.%m.%Y"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, *kwargs)
        self.columns = None
        self.mappings = None

    def split_records(self):
        return csv.reader(self.fin, delimiter=';', quotechar='"')

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """

        # First line of CSV file contains headers, not an actual transaction
        if self.cur_record == 1:
            # Prepare columns headers lookup table for parsing
            # v ... column heading
            # i ... column index (expected by .mappings)
            self.columns = {v: i for i,v in enumerate(line)}
            # .date_user cannot be parsed by super().parse_record(line)
            # because of different date_format_user
            self.mappings = {
                "date":      self.columns['Datum zaúčtování'],
                "memo":      self.columns['Zpráva'],
                "payee":     self.columns['Název protiúčtu'],
                "amount":    self.columns['Zaúčtovaná částka'],
                "check_no":  self.columns['VS'],
                "refnum":    self.columns['Id transakce'],
            }

            # And skip further processing by parser
            return None

        # Shortcut
        columns = self.columns

        # Normalize string
        for i,v in enumerate(line):
            line[i] = v.strip()

        if line[columns["Zaúčtovaná částka"]] == "":
            line[columns["Zaúčtovaná částka"]] = "0"

        if line[columns["Poplatek"]] == "":
            line[columns["Poplatek"]] = "0"

        StatementLine = super(RaiffeisenCZParser, self).parse_record(line)

        StatementLine.id = statement.generate_transaction_id(StatementLine)

        if not line[columns["Datum provedení"]] == "":
            StatementLine.date_user = datetime.strptime(line[columns["Datum provedení"]], self.date_format_user)

        if   line[columns["Typ transakce"]].startswith("Převod"):
            StatementLine.trntype = "XFER"
        elif line[columns["Typ transakce"]].startswith("Platba"):
            StatementLine.trntype = "XFER"
        elif line[columns["Typ transakce"]].startswith("Jednorázová platba"):
            StatementLine.trntype = "XFER"
        elif line[columns["Typ transakce"]].startswith("Příchozí platba"):
            StatementLine.trntype = "CREDIT"
        elif line[columns["Typ transakce"]].startswith("Trvalý převod"):
            StatementLine.trntype = "REPEATPMT"
        elif line[columns["Typ transakce"]].startswith("Trvalá platba"):
            StatementLine.trntype = "REPEATPMT"
        elif line[columns["Typ transakce"]].startswith("Kladný úrok"):
            StatementLine.trntype = "INT"
        elif line[columns["Typ transakce"]].startswith("Záporný úrok"):
            StatementLine.trntype = "INT"
        elif line[columns["Typ transakce"]].startswith("Inkaso"):
            StatementLine.trntype = "DIRECTDEBIT"
        elif line[columns["Typ transakce"]].startswith("Srážka daně"):
            StatementLine.trntype = "DEBIT"
        elif line[columns["Typ transakce"]].startswith("Daň z úroků"):
            StatementLine.trntype = "DEBIT"
        elif line[columns["Typ transakce"]].startswith("Správa účtu"):
            StatementLine.trntype = "FEE"
        elif line[columns["Typ transakce"]].startswith("Jiný trans."):
            StatementLine.trntype = "FEE"
        elif line[columns["Typ transakce"]].startswith("Správa účtu"):
            StatementLine.trntype = "FEE"
        elif line[columns["Typ transakce"]].startswith("Poplatek"):
            StatementLine.trntype = "FEE"
        elif line[columns["Typ transakce"]].startswith("Směna"):
            StatementLine.trntype = "FEE"
        elif line[columns["Typ transakce"]].startswith("Zpráva"):
            StatementLine.trntype = "FEE"

        # Add payee's account number to payee field
        StatementLine.payee = StatementLine.payee + "|" + line[columns["Číslo protiúčtu"]]
        StatementLine.payee = StatementLine.payee.strip("|")

        # Add payment symbols to memo field
        if line[columns["Poznámka"]] != "" and line[columns["Poznámka"]] != StatementLine.memo:
            StatementLine.memo = StatementLine.memo + ", " + line[columns["Poznámka"]]
            StatementLine.memo = StatementLine.memo.strip(", ")

        if line[columns["VS"]] != "":
            StatementLine.memo = StatementLine.memo + "|VS:" + line[columns["VS"]]
        if line[columns["KS"]] != "":
            StatementLine.memo = StatementLine.memo + "|KS:" + line[columns["KS"]]
        if line[columns["SS"]] != "":
            StatementLine.memo = StatementLine.memo + "|SS:" + line[columns["SS"]]

        StatementLine.memo = StatementLine.memo.strip("|")

        # Raiffeisen may show various fees on the same line as the underlying transaction.
        # In case there is a fee connected with the transaction, the fee is added as different transaction

        # It may include thousands separators
        # ToDo: re-use parse_float (how??)
        line[columns["Poplatek"]] = re.sub(",", ".", line[columns["Poplatek"]])
        line[columns["Poplatek"]] = re.sub("[ a-zA-Z]", "", line[columns["Poplatek"]])

        # Some type of fee is standalone, not related to transaction amount. Add it to amount field only
        if float(line[columns["Poplatek"]]) != 0 and StatementLine.amount == 0:
            StatementLine.amount = float(line[columns["Poplatek"]])

        # Duplicate the current line and replace .amount with the fee amount ["Poplatek"]
        elif float(line[columns["Poplatek"]]) != 0 and StatementLine.amount != 0:
            fee_line = line[:]
            fee_line[columns["Zaúčtovaná částka"]] = line[columns["Poplatek"]]
            fee_line[columns["Poplatek"]] = ""
            fee_line[columns["Typ transakce"]] = "Poplatek"
            fee_line[columns["Zpráva"]] = "Poplatek: " + fee_line[columns["Zpráva"]]

            # Parse the newly generated fee_line and append it to the rest of the statements
            stmt_line = self.parse_record(fee_line)
            if stmt_line:
                stmt_line.assert_valid()
                self.statement.lines.append(stmt_line)

        if StatementLine.amount == 0:
            return None

        return StatementLine

    # The numbers in CSV may include some non-numerical chars, remove them.
    def parse_float(self, value):
        value = re.sub(",", ".", value)
        value = re.sub("[ a-zA-Z]", "", value)
        return super(RaiffeisenCZParser, self).parse_float(value)



import csv
import sys
import itertools

from ofxstatement import statement
from ofxstatement.plugin import Plugin
from ofxstatement.parser import CsvStatementParser
from ofxstatement.parser import StatementParser
from ofxstatement.statement import StatementLine

class IngDePlugin(Plugin):
    """ING DiBa Germany / Deutschland Plugin
    """

    def get_parser(self, filename):
        f = open(filename, 'r', encoding=self.settings.get("charset", "ISO-8859-1"))
        parser = IngDeParser(f)
        return parser

class IngDeParser(CsvStatementParser):

    date_format = "%d.%m.%Y"
    mappings = {
        #'date_user':  0, # doesn't work for some reason
        'date':       1,
        'payee':      2,
        'memo':       4,
        'amount':     7
    }

    reader = None
    currency = None
    
    def parse(self):
        """Main entry point for parsers

        super() implementation will call to split_records and parse_record to
        process the file.
        """

        # initialize csv reader
        self.reader = csv.reader(self.fin, delimiter=';')

        # skip first three lines
        self.reader = itertools.islice(self.reader, 3, None)

        # get bank / account details
        account_id = self.reader.__next__()[1]     # account id = IBAN
        germantype = self.reader.__next__()[1]     # account type (german name)
        bank_id    = self.reader.__next__()[1]     # bank id
        customer   = self.reader.__next__()[1]     # customer name
        timespan   = self.reader.__next__()[1]     # timespan

        # check wether saldo is included in csv file
        try:
            saldo = self.reader.__next__()[1]      # initial saldo
        except IndexError:
            raise RuntimeError('Please export CSV with Saldo!')


        # skip another 6 lines
        self.reader = itertools.islice(self.reader, 6, None)

        # parse each line
        stmt = super(IngDeParser, self).parse()

        # fill in bank infos
        stmt.account_id = account_id.replace(" ", "")
        stmt.bank_id = bank_id
        stmt.currency = self.currency

        # try to get account type
        if "Girokonto" == germantype:
            stmt.account_type = "CHECKING"
        elif "Extra-Konto" == germantype:
            stmt.account_type = "SAVINGS"
        
        # finalize statement
        statement.recalculate_balance(stmt)

        return stmt

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        return self.reader

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        saldo =          str(line[5])
        currency_saldo = str(line[6])
        currency_trans = str(line[8])

        # check currency
        if None == self.currency:
            self.currency = currency_saldo
        elif self.currency != currency_trans and self.currency != currency_saldo:
            raise ValueError('Different currencies are not supported!')

        # fix german number format
        line[7] = line[7].replace(".","").replace(",",".") 
        
        # parse line elements using the mappings defined above (call parse_record() from parent class)
        stmtline = super(IngDeParser, self).parse_record(line)

        # check if debit or credit
        stmtline.trntype = 'DEBIT' if stmtline.amount < 0 else 'CREDIT'

        # generate id for statement
        id_date = stmtline.date.strftime('%Y%m%d')

        # create a hash from payee, memo, amount, saldo
        id_hash = str(hash(stmtline.payee + stmtline.memo + str(stmtline.amount) + saldo) % ((sys.maxsize + 1) * 2))

        # final id is constructed from date and hash (so hopefully this is unique)
        stmtline.id = id_date + id_hash

        return stmtline



# Transaction type mapping for Nordea statement parsers

TRANSACTION_TYPES = {
   "TILISIIRTO": "XFER",
   "PALVELUMAKSU": "FEE",
   "PKORTTIMAKSU": "POS",
   "VIITESIIRTO": "XFER",
   "PIKASIIRTO": "XFER",
   "ETUUS": "XFER",
   "AUTOMAATTINOSTO": "ATM",
   "MAKSUPALVELU": "REPEATPMT",
   "TALLETUSKORKO": "INT",
   "LÄHDEVERO": "FEE",
   "SUORAVELOITUS": "DIRECTDEBIT",
   "PANO": "DEP",
   "VUOKRA": "PAYMENT",
   "VERONPALAUTUS": "DIRECTDEP",
   "PALKKA": "DIRECTDEP",
   "VISA ELECTRON-OSTO": "DIRECTDEBIT", # Added for Nordea
   "ATMOTTO/OTTO.": "ATM", # Added for Nordea
   "PALVELUMAKSU ALV 0%": "FEE",
   "ITSEPALVELU": "XFER",
   "E-MAKSU": "PAYMENT",
   "E-LASKU": "PAYMENT",
}

#   Other types available: 'CREDIT', 'DEBIT', 'INT', 'DIV', 'DEP',
#   'CHECK', 'CASH', 'OTHER'





#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.

import datetime
from decimal import Decimal
import os
import unittest
from ofxstatement.statement import generate_transaction_id

from ofxstatement.plugins.raiffeisen import RaiffeisenCsvParser


class TestRaiffeisenCsvParser(unittest.TestCase):
    """Unit tests for RaiffeisenCsvParser."""

    def setUp(self):
        csvfile = os.path.join(
            os.path.dirname(__file__), 'samples', 'raiffeisen.csv')
        with open(csvfile, 'r', encoding='cp1252') as fin:
            self.statement = RaiffeisenCsvParser(fin).parse()

    def test_statement_properties(self):
        self.assertEqual(len(self.statement.lines), 7)
        self.assertEqual(self.statement.start_balance, 0.0)
        self.assertAlmostEqual(self.statement.end_balance, Decimal('-157.89'))
        self.assertEqual(self.statement.currency, "EUR")
        self.assertEqual(
            self.statement.start_date, datetime.datetime(2013, 6, 28, 0, 0))
        self.assertEqual(
            self.statement.end_date, datetime.datetime(2013, 7, 4, 0, 0))

    def test_line0_interest_earned(self):
        line = self.statement.lines[0]
        self.assertEqual(line.amount, Decimal('0.58'))
        self.assertEqual(line.memo, "0,125 % p.a. Habenzinsen ab 01.04.13")
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2013, 6, 28, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line1_interest_paid(self):
        line = self.statement.lines[1]
        self.assertEqual(line.amount, Decimal('-0.15'))
        self.assertEqual(line.memo, "Kapitalertragsteuer")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2013, 6, 28, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line2_service_fee_print_statement(self):
        line = self.statement.lines[2]
        self.assertEqual(line.amount, Decimal('-0.11'))
        self.assertEqual(line.memo, "Entgelt Kontoauszug")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2013, 6, 28, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line3_service_fee(self):
        line = self.statement.lines[3]
        self.assertEqual(line.amount, Decimal('-6.65'))
        self.assertEqual(line.memo, "Entgelt Kontoführung")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2013, 6, 28, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line4_elba_payment(self):
        line = self.statement.lines[4]
        self.assertEqual(line.amount, Decimal('-175.16'))
        self.assertEqual(
            line.memo, "ELBA-INTERNET VOM 29.06 UM 09:16 Empfänger: A person "
            "Verwendungszweck: Invoice number 10")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2013, 7, 1, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line5_debit(self):
        line = self.statement.lines[5]
        self.assertEqual(line.amount, Decimal('-100'))
        self.assertEqual(
            line.memo, "Lastschrift Auftraggeber: A company "
            "Kundendaten: 000000000000 111111111111 Verwendungszweck: reason")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2013, 7, 1, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line6_credit(self):
        line = self.statement.lines[6]
        self.assertEqual(line.amount, Decimal('123.60'))
        self.assertEqual(
            line.memo, "Gutschrift Auftraggeber: A person Kundendaten: reason")
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2013, 7, 4, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



#!/usr/bin/python3
"""Setup
"""
from setuptools import find_packages
from distutils.core import setup

version = "0.5.1.dev0"

with open("README.rst") as f:
    long_description = f.read()

setup(
    name="ofxstatement-iso20022",
    version=version,
    author="Andrey Lebedev",
    author_email="andrey@lebedev.lt",
    url="https://github.com/kedder/ofxstatement-iso20022",
    description=("ISO-20022 plugin for ofxstatement"),
    long_description=long_description,
    license="GPLv3",
    keywords=["ofx", "banking", "statement", "iso-20022"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "Natural Language :: English",
        "Topic :: Office/Business :: Financial :: Accounting",
        "Topic :: Utilities",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU Affero General Public License v3",
    ],
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["ofxstatement", "ofxstatement.plugins"],
    entry_points={
        "ofxstatement": ["iso20022 = ofxstatement.plugins.iso20022:Iso20022Plugin"]
    },
    install_requires=["ofxstatement"],
    extras_require={"test": ["pytest"]},
    include_package_data=True,
    zip_safe=True,
)



#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.

import datetime
import os
import unittest
from ofxstatement.statement import generate_transaction_id

from ofxstatement.plugins.livebank import LivebankCsvParser


class TestLivebankCsvParser(unittest.TestCase):
    """Unit tests for LivebankCsvParser."""

    def setUp(self):
        csvfile = os.path.join(
            os.path.dirname(__file__), 'samples', 'livebank.csv')
        with open(csvfile, 'r', encoding='iso-8859-1') as fin:
            self.statement = LivebankCsvParser(fin).parse()

    def test_statement_properties(self):
        self.assertEqual(len(self.statement.lines), 3)
        self.assertEqual(self.statement.start_balance, 0.0)
        self.assertAlmostEqual(self.statement.end_balance, 5050)
        self.assertEqual(self.statement.currency, "EUR")
        self.assertEqual(self.statement.account_id, "12345678")
        self.assertEqual(
            self.statement.start_date, datetime.datetime(2013, 6, 5, 0, 0))
        self.assertEqual(
            self.statement.end_date, datetime.datetime(2013, 7, 3, 0, 0))

    def test_line0_credit(self):
        line = self.statement.lines[0]
        self.assertEqual(line.amount, 150.00)
        self.assertEqual(line.memo, "Datenträger-Umsatz")
        self.assertEqual(
            line.payee, "A name, A text, REF: XXXXXXXXXXXXXXXXXXXXXXXXXXXX")
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2013, 7, 3, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line1_debit(self):
        line = self.statement.lines[1]
        self.assertEqual(line.amount, -100)
        self.assertEqual(line.memo, "Internetauftrag")
        self.assertEqual(
            line.payee, "A text, A reference, A text")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2013, 6, 10, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line2_credit(self):
        line = self.statement.lines[2]
        self.assertEqual(line.amount, 5000)
        self.assertEqual(line.memo, "Datenträger-Umsatz")
        self.assertEqual(
            line.payee, "A name, A text, REF: XXXXXXXXXXXXXXXXXXXXXXXXXXXX")
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2013, 6, 5, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



import csv
import sys
from datetime import datetime
from decimal import Decimal, InvalidOperation

import hashlib
import re
from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import StatementLine, Statement

__author__ = 'Chris Mayes'
__email__ = 'cmayes@cmay.es'
__version__ = '0.3.1'


def warning(*objs):
    print("WARNING: ", *objs, file=sys.stderr)


class BettermentPlugin(Plugin):
    """Plugin for Betterment CSV statements.

    Note that you can specify 'bank' and 'account' in ofxstatement's configuration file (accessible
    using the `ofxstatement edit-config` command or directly at
    `~/.local/share/ofxstatement/config.ini` (on Linux, at least).  Setting these values under the
    "betterment section" (i.e. "[betterment]") makes it easier for your personal finance application
    to recognize which account the file's data belongs to.

    Also note that transactions for zero amounts are filtered by default.  If you wish to include
    zero-amount transactions, set 'zero_filter' to 'false' in your settings.
    """

    def get_parser(self, fin):
        parser = BettermentParser(fin, self.settings.get('charset', 'utf-8'))
        parser.statement.bank_id = self.settings.get('bank', 'Betterment')
        parser.statement.account_id = self.settings.get('account', None)
        parser.statement.filter_zeros = str2bool(self.settings.get('filter_zeros', "true"))
        return parser


def process_balances(stmt):
    """
    Based on `ofxstatement.statement.recalculate_balance`. Uses the "Ending Balance" field
    from the Betterment transaction data for fetching the start and end balance for this statement.
    If the statement has no transactions, it is returned unmodified.

    :param stmt: The statement data to process.
    :return: Whether the balances were calculated.
    """
    if not stmt.lines:
        return False
    date_sorted_lines = sorted(stmt.lines, key=lambda k: k.date)
    first_line = date_sorted_lines[0]
    stmt.start_balance = (first_line.end_balance - first_line.amount)
    stmt.start_date = first_line.date
    last_line = date_sorted_lines[-1]
    stmt.end_balance = last_line.end_balance
    stmt.end_date = last_line.date
    return True


class BettermentParser(StatementParser):
    mappings = {"date": "Date Completed",
                "memo": "Transaction Description",
                "amount": "Amount"
                }
    date_format = "%Y-%m-%d %H:%M:%S %z"
    old_date_format = "%Y-%m-%d %H:%M:%S.%f"

    def __init__(self, fin, encoding):
        self.statement = Statement()
        self.fin = fin
        self.encoding = encoding

    def parse(self):
        """Read and parse statement

        Return Statement object

        May raise exceptions.ParseException on malformed input.
        """
        with open(self.fin, 'r', encoding=self.encoding) as fhandle:
            for line in csv.DictReader(fhandle):
                self.cur_record += 1
                if not line:
                    continue
                stmt_line = self.parse_record(line)
                if stmt_line:
                    stmt_line.assert_valid()
                    self.statement.lines.append(stmt_line)
            process_balances(self.statement)
            return self.statement

    def parse_record(self, line):
        san_line = {}
        for lkey, lval in line.items():
            san_line[lkey] = re.sub("\$", "", lval)

        stmt_line = StatementLine()
        for field, col in self.mappings.items():
            rawvalue = san_line[col]
            try:
                value = self.parse_value(rawvalue, field)
            except ValueError as e:
                # Invalid data line; skip it
                warning("Error parsing value '{}' on line '{}': {}".format(rawvalue, san_line, e))
                return None
            setattr(stmt_line, field, value)

        if self.statement.filter_zeros and stmt_line.amount == 0:
            return None

        try:
            stmt_line.end_balance = Decimal(san_line['Ending Balance'])
        except InvalidOperation:
            # Usually indicates a pending transaction
            return None

        # generate transaction id out of available data
        stmt_line.id = generate_stable_transaction_id(stmt_line)
        return stmt_line

    def parse_datetime(self, value):
        try:
            return datetime.strptime(value, self.date_format)
        except ValueError:
            return datetime.strptime(value, self.old_date_format)


def str2bool(v):
    """Converts a string to a boolean value.  From http://stackoverflow.com/a/715468"""
    return v.lower() in ("yes", "true", "t", "1")


def generate_stable_transaction_id(stmt_line):
    """Generates a stable, pseudo-unique id for given statement line.

    This function can be used in statement parsers when a real transaction id is
    not available in the source statement.
    """
    h = hashlib.sha256()
    if stmt_line.date is not None:
        h.update(str(stmt_line.date).encode('utf-8'))
    if stmt_line.memo is not None:
        h.update(stmt_line.memo.encode('utf-8'))
    if stmt_line.amount is not None:
        h.update(str(stmt_line.amount).encode('utf-8'))
    return h.hexdigest()



#    SberBank (https://www.sberbank.ru/) CSV plugin for ofxstatement
#
#    Copyright 2013 Andrey Lebedev <andrey@lebedev.lt>
#    Copyright 2016 Alexander Gerasiov <gq@cs.msu.su>
#    Copyright 2020 Dmitry Pavlov <zeldigas@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3 as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import csv
from decimal import Decimal

from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement import statement
from datetime import datetime

# file format options
SB_DELIMITER = ';'
SD_TIME_FORMAT = '%d.%m.%Y'
SD_ENCODING = 'utf-8'
SB_FIELDNAMES = ['card_type', 'card_num', 'date_user', 'date', 'auth_code', 'op_type', 'op_city',
                 'op_country', 'description', 'currency', 'currency_amount', 'amount']


class SberBankCSVStatementParser(StatementParser):
    statement = None

    def __init__(self, fin):
        super().__init__()
        self.statement = statement.Statement()
        self.fin = fin
        # Skip 1st row with column's headers
        self.fin.readline()
        self.cur_record = 1

    def split_records(self):
        return csv.DictReader(self.fin, delimiter=SB_DELIMITER, fieldnames=SB_FIELDNAMES)

    def parse_record(self, line):
        transaction = statement.StatementLine()

        if not self.statement.account_id:
            self.statement.account_id = '{} {}'.format(line['card_type'], line['card_num'])

        transaction.date = datetime.strptime(line['date'], SD_TIME_FORMAT)
        transaction.date_user = datetime.strptime(line['date_user'], SD_TIME_FORMAT)

        transaction.amount = Decimal(line['amount'].replace(',', '.'))

        transaction.trntype = 'DEBIT' if transaction.amount > 0 else 'CREDIT'

        transaction.memo = ', '.join(line[f] for f in
                                     ('description', 'op_city', 'op_country', 'op_type') if line[f])

        # as csv file does not contain explicit id of transaction, generating artificial one
        transaction.id = statement.generate_transaction_id(transaction)

        return transaction


class SberBankCSVPlugin(Plugin):
    """SberBank CSV (http://sberbank.ru)
    """

    def get_parser(self, fin):
        f = open(fin, 'r', encoding=SD_ENCODING)
        parser = SberBankCSVStatementParser(f)
        parser.statement.currency = self.settings.get('currency')
        parser.statement.account_id = self.settings.get('account')
        parser.statement.bank_id = self.settings.get('bank', 'SberBank')
        return parser



#    SberBankTxt (https://www.sberbank.ru/) plugin for ofxstatement
#
#    Copyright 2013 Andrey Lebedev <andrey@lebedev.lt>
#    Copyright 2016 Alexander Gerasiov <gq@cs.msu.su>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3 as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement import statement
from datetime import datetime
import re

# file format options
sb_encoding = 'cp1251'


class ParserState:
    name = ""
    matchers = None

    def __init__(self, name, parser):
        self.name = name
        self.matchers = list()
        parser.append(self)

    def addMatcher(self, reString, nextState=None, function=None):
        self.matchers.append([re.compile(reString), nextState, function])

    def run(self, line):
        for (matcher, nextState, function) in self.matchers:
            match = matcher.match(line)
            if match:
                if function:
                    function(match)
                return nextState
        return None

    def __str__(self):
        string = "State '%s' matchers:\n" % self.name
        for matcher in self.matchers:
            string += "re '%s' => state '%s'\n" % (matcher[0], matcher[1])
        return string


class SberBankTxtStatementParser(StatementParser):
    statement = None

    transaction = None
    account_id = ""
    account_fl_len = 0

    machine = {}
    currentState = None
    internal = None

    def append(self, state):
        self.machine[state.name] = state

    def extractCurrency(self, match):
        if not self.statement.currency:
            self.statement.currency = match.group(1)

    def extractBeginBalance(self, match):
        if not self.statement.start_balance:
            self.statement.start_balance = float(match.group(1))

    def extractEndBalance(self, match):
        if not self.statement.end_balance:
            self.statement.end_balance = float(match.group(1))
        if not self.statement.account_id:
            self.statement.account_id = " ".join(self.account_id.split())
        if self.transaction:
            self.transaction.memo = " ".join(self.transaction.memo.split())
            self.statement.lines.append(self.transaction)
            self.transaction = None

    def parseDate(self, string):
        rusMonths = {
            u'ЯНВ': 1,
            u'ФЕВ': 2,
            u'МАР': 3,
            u'АПР': 4,
            u'МАЙ': 5,
            u'ИЮН': 6,
            u'ИЮЛ': 7,
            u'АВГ': 8,
            u'СЕН': 9,
            u'ОКТ': 10,
            u'НОЯ': 11,
            u'ДЕК': 12,
        }
        return datetime(2000 + int(string[5:]), rusMonths[string[2:5]], int(string[:2]))

    def extractTransaction(self, match):
        if self.transaction:
            self.transaction.memo = " ".join(self.transaction.memo.split())
            self.statement.lines.append(self.transaction)

        self.transaction = statement.StatementLine()

        self.account_fl_len = len(match.group(1))

        self.transaction.date = self.parseDate(match.group(3))
        self.transaction.memo = match.group(4)
        self.transaction.amount = float(match.group(5)) * (1 if match.group(6) else -1)
        self.transaction.trntype = 'DEBIT' if match.group(6) else 'CREDIT'
        if match.group(1).strip():
            self.account_id += match.group(1)

    def extractTransactionAppend(self, match):
        first = match.group(1)[:self.account_fl_len]
        second = match.group(1)[self.account_fl_len:]
        self.account_id += first
        self.transaction.memo += second

    def __init__(self, fin):
        self.statement = statement.Statement()
        self.internal = {}
        self.fin = fin

        self.currentState = 'init'

        state = ParserState('init', self)
        state.addMatcher(r"^.*ВАЛЮТА СЧЕТА.*$", 'currency')

        state = ParserState('currency', self)
        state.addMatcher(r"^\s*(\w{3})\s*$",
                         'begin_balance',
                         self.extractCurrency)

        state = ParserState('begin_balance', self)
        state.addMatcher(r"^ОСТАТОК НА НАЧАЛО ПЕРИОДА:\s*(\d+\.\d{2})(\+)?\s*$",
                         'table_header',
                         self.extractBeginBalance)

        state = ParserState('table_header', self)
        state.addMatcher(r"^[-+]{80,}$", 'table_header2')

        state = ParserState('table_header2', self)
        state.addMatcher(r"^[-+]{80,}$", 'transaction')

        state = ParserState('transaction', self)
        state.addMatcher(r"^[-+]{80,}$", 'end_balance')
        state.addMatcher(
            r"^(.*)\s*(\d{2}[А-Я]{3})\s+(\d{2}[А-Я]{3}\d{2})\s+\d{6}\s+(.*)\s\w{3}\s+\d*\.\d{2}\s+(\d*\.\d{2})(CR)?\s*$",
            None,
            self.extractTransaction)
        state.addMatcher(
            r"^(.*)\s*(\d{2}[А-Я]{3})\s+(\d{2}[А-Я]{3}\d{2})\s+\d{6}\s+(КОМИССИЯ)\s+(\d*\.\d{2})(CR)?\s*$",
            None,
            self.extractTransaction)
        state.addMatcher(
            r"^(.*)\s*(\d{2}[А-Я]{3})\s+(\d{2}[А-Я]{3}\d{2})\s+\d{6}\s+(.*)\s(\d*\.\d{2})(CR)?\s*$",
            None,
            self.extractTransaction)
        state.addMatcher(r".*ИТОГО ПО.*")
        state.addMatcher(r"^(.+)\s*$",
                         None,
                         self.extractTransactionAppend)

        state = ParserState('end_balance', self)
        state.addMatcher(r"^ОСТАТОК НА КОНЕЦ ПЕРИОДА:\s*(\d+\.\d{2})\+?\s*$",
                         'table_header',
                         self.extractEndBalance)

    def run(self, line):
        nextState = self.machine[self.currentState].run(line)
        if nextState:
            self.currentState = nextState

    def parse(self):
        for line in self.fin:
            self.run(line)

        return self.statement


class SberBankTxtPlugin(Plugin):
    """SberBank TXT (http://sbrf.ru)
    """

    def get_parser(self, fin):
        f = open(fin, 'r', encoding=sb_encoding)
        parser = SberBankTxtStatementParser(f)
        parser.statement.currency = self.settings.get('currency', None)
        parser.statement.account_id = self.settings.get('account', None)
        parser.statement.bank_id = self.settings.get('bank', 'SberBank')
        return parser



#!/usr/bin/python3
"""Setup
"""

import os

from setuptools import find_packages
from setuptools.command.test import test as TestCommand
from distutils.core import setup

import unittest

version = "0.1.1"

with open('README.md') as f:
    long_description = f.read()

setup(name='ofxstatement-be-ing',
      version=version,
      author="Bernardo Bandos",
      author_email="@hotmail.com",
      url="https://github.com/jbbandos/ofxstatement-be-ing",
      description=("OFXStatement plugin for ING (Belgium)"),
      long_description=open("README.md").read(),
      long_description_content_type='text/markdown',
      license="GPLv3",
      keywords=["ofx", "banking", "statement"],
      classifiers=[
          'Development Status :: 4 - Beta',
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'Topic :: Office/Business :: Financial :: Accounting',
          'Topic :: Utilities',
          'Environment :: Console',
          'Operating System :: OS Independent',
          'License :: OSI Approved :: GNU Affero General Public License v3'],
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=["ofxstatement", "ofxstatement.plugins"],
      entry_points={
          'ofxstatement':
          ['ingbe = ofxstatement.plugins.ingbe:IngBePlugin']
          },
      install_requires=['ofxstatement'],
      extras_require={'test': ["freezegun", "pytest"]},
      include_package_data=True,
      zip_safe=True
      )



import os
import re
import datetime
from decimal import Decimal

from ofxstatement.plugins.danske import DanskeCsvStatementParser


def test_DanskeCsvStatementParser() -> None:
    # Test DanskeCsvStatementParser

    # Open sample csv to parse
    csvfile = os.path.join(os.path.dirname(__file__), "samples", "danske.csv")

    # Create parser and parse
    fin = open(csvfile, "r", encoding="cp1257")
    parser = DanskeCsvStatementParser(fin)
    statement = parser.parse()

    # Check what we've got:
    assert len(statement.lines) == 4
    assert statement.start_balance == Decimal("0")
    assert statement.end_balance == Decimal("0.00")
    assert statement.start_date == datetime.datetime(2012, 3, 1, 0, 0)
    assert statement.end_date == datetime.datetime(2012, 3, 7, 0, 0)

    # First line is a payment for incoming transaction:
    line = statement.lines[0]
    assert line.amount == Decimal("-7.8")
    assert line.memo == (
        "Paslaugų ir komisinių pajamos už gaunamus tarptautinius pervedimus USD"
    )
    assert line.date == datetime.datetime(2012, 3, 1, 0, 0)
    assert line.id == "bef3396a4c86263e911f11a3b10bcc5558064116"

    # Second line is incoming money
    line = statement.lines[1]
    assert line.amount == Decimal("889.81")
    assert line.memo == "ACME LLC"
    assert line.date == datetime.datetime(2012, 3, 1, 0, 0)

    # Third line is a pament with amount less than 1 USD
    line = statement.lines[2]
    assert line.amount == Decimal("-0.46")
    assert line.memo is not None
    assert re.match(
        r"Mokestis už lėšų, gautų iš kitų LR registruotų bankų, .* 2012\.03\.06",
        line.memo,
    )

    # Fourth is a transfer to other account
    line = statement.lines[3]
    assert line.memo == "John Doe"
    assert line.amount == Decimal("-881.55")



import csv

from ofxstatement.plugin import Plugin
from ofxstatement.parser import CsvStatementParser
from ofxstatement.statement import Statement, StatementLine, BankAccount, recalculate_balance


class TriodosBePlugin(Plugin):
    """Belgian Triodos plugin for ofxstatement
    """

    def get_parser(self, filename):
        f = open(filename, 'r', encoding=self.settings.get("charset", "ISO-8859-1"))
        parser = TriodosBeParser(f)
        return parser


class TriodosBeParser(CsvStatementParser):
    date_format = "%d-%m-%Y"
    mappings = {
        'date': 0,
        'payee': 5,
        'memo': 8,
        'amount': 2
    }

    def __init__(self, filename):
        self.statement = Statement('TRIOBEBB', None,'EUR')
        self.fin = filename
        self.statement.currency = 'EUR'

    def parse(self):
        """Main entry point for parsers

        super() implementation will call to split_records and parse_record to
        process the file.
        """
        stmt = super(TriodosBeParser, self).parse()
        recalculate_balance(stmt)
        return stmt

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        return csv.reader(self.fin)

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        if (self.statement.account_id == None):
            self.statement.account_id =  line[1]
        stmtline = super(TriodosBeParser, self).parse_record(line)
        stmtline.trntype = 'DEBIT' if stmtline.amount < 0 else 'CREDIT'
        stmtline.bank_account_to = BankAccount(line[4], line[3])
        return stmtline

    def parse_float(self, value):
        """Return a float from a string with ',' as decimal mark.
         ex. 1.234,56 -> 1234.56
        """
        return float(value.replace('.','').replace(',', '.'))
